// Dialog_Options_Dictionaries.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_DICTIONARIES_H
#define DIALOG_OPTIONS_DICTIONARIES_H
///////////////////////////////////////////////////////////
#include "Dialog_ChangeDescription.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options_Dictionaries( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// Check the indexing button:
			if( registry.GetBOOL(".\\Dictionaries\\Indexing") ) {
				SendDlgItemMessage( hWnd, IDC_DICTIONARIES_INDEXING, BM_SETCHECK, BST_CHECKED, 0 );
			}

			// Populate the list:
			HWND hwndList = GetDlgItem( hWnd, IDC_DICTIONARIES_LIST );
			for( int i = 0; i < edict_dictionaries.size(); i++ ) {
				SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM)ToWString(edict_dictionaries[i]->displayname).c_str() ); 
				SendMessage(hwndList, LB_SETITEMDATA, i, (LPARAM)i ); 
			}

			return(TRUE);
		}
		case WM_COMMAND: {
			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				case IDC_DICTIONARIES_INDEXING: {
					switch( code ) {
						case BN_CLICKED: {
							if( IsDlgButtonChecked( hWnd, IDC_DICTIONARIES_INDEXING ) ) {
								registry.SetBOOL(".\\Dictionaries\\Indexing", true );
							} else {
								registry.SetBOOL(".\\Dictionaries\\Indexing", false );
							}
							break;
						}
					}
					break;
				}
				case IDC_DICTIONARIES_CHANGEDESCRIPTION: {
					switch( code ) {
						case BN_CLICKED: {
							int dict = SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_GETCURSEL, 0,0 );
							if( dict != LB_ERR ) {
								int result = CreateDialog_ChangeDescription( hWnd, edict_dictionaries[dict] );
								if( result == IDOK ) {
									SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_DELETESTRING, dict,0 );
									SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_INSERTSTRING, dict,(LPARAM)ToWString(edict_dictionaries[dict]->displayname).c_str() );
									SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_SETITEMDATA, dict, (LPARAM)dict ); 
									SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_SETCURSEL, dict, 0 );

									// Update the registry:
									registry.CD(".\\Dictionaries\\EDICT");
										char path[35]; sprintf(path,"%i",dict );
										registry.CD( path );
											registry.SetSTRING( "Name", edict_dictionaries[dict]->displayname.c_str() );
										registry.CD("..");
									registry.CD("..");
								}								
							}
							break;
						}
					}
					break;
				}
				case IDC_DICTIONARIES_DELETE: {
					switch( code ) {
						case BN_CLICKED: {
							int dict = SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_GETCURSEL, 0,0 );
							if( dict != LB_ERR ) {
								// Delete dictionary #dict:

								if( MessageBox(hWnd,L"Are you sure that you want to delete this dictionary?",L"Warning",MB_YESNO ) == IDYES ) {
									// Update the registry:
									registry.CD(".\\Dictionaries\\EDICT");
										for( int d = dict; d < edict_dictionaries.size()-1; d++ ) {
											char path[35]; sprintf(path,"%i",dict );
											registry.CD( path );
												registry.SetSTRING( "Path", edict_dictionaries[dict+1]->filename.c_str() );
												registry.SetSTRING( "Name", edict_dictionaries[dict+1]->displayname.c_str() );
												registry.SetBOOL( "On", edict_dictionaries[dict+1]->on );
											registry.CD("..");
										}

										// Get rid of the last one:
										char path[35]; sprintf(path,"%i",edict_dictionaries.size()-1 );
										registry.DeleteKey( path );

										edict_dictionaries.erase( edict_dictionaries.begin()+dict );

										// Set the number:
										registry.SetINT( "Number", edict_dictionaries.size() );
									registry.CD("..");

									SendDlgItemMessage( hWnd, IDC_DICTIONARIES_LIST, LB_DELETESTRING, dict,0 );
								}
							}
							break;
						}
					}
					break;
				}
			}
		}
   }
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateDialog_Options_Dictionaries( HWND hwndParent ) {
	HWND hwnd = CreateDialog(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS_DICTIONARIES ), 
		hwndParent,
		(DLGPROC)DlgProc_Options_Dictionaries
	);
	return( hwnd );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
