// Dialog_Options_JLookUp.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef DIALOG_OPTIONS_JLOOKUP_H
#define DIALOG_OPTIONS_JLOOKUP_H
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
BOOL CALLBACK DlgProc_Options_JLookUp( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	switch( message ) {
		case WM_INITDIALOG: {
		/* wParam
				Handle to the control to receive the default keyboard focus. The system assigns the default keyboard focus only if the dialog box procedure returns TRUE. 
			lParam
				Specifies additional initialization data. This data is passed to the system as the lParam parameter in a call to the CreateDialogIndirectParam, CreateDialogParam, DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box. For property sheets, this parameter is a pointer to the PROPSHEETPAGE structure used to create the page. This parameter is zero if any other dialog box creation function is used. 
			Return: TRUE if we want to set keyboard focus to wParam, FALSE otherwise.
		*/

			// [x] Remember window size and position:
			if( registry.GetBOOL(".\\Window\\RememberSize") ) {
				SendDlgItemMessage( hWnd, IDC_JLOOKUP_REMEMBERSIZE, BM_SETCHECK, BST_CHECKED, 0 );
			} else {
				SendDlgItemMessage( hWnd, IDC_JLOOKUP_REMEMBERSIZE, BM_SETCHECK, BST_UNCHECKED, 0 );
			}

			// [ x#### ] [ y#### ] [Set]:
			RECT rcWin;
				GetWindowRect( hwnd_mainwindow, &rcWin );
			Uint16 str[32];
				swprintf(str,L"%i",rcWin.left );
			SendDlgItemMessage( hWnd, IDC_JLOOKUP_WINDOW_X, WM_SETTEXT, 0, (LPARAM)str );
				swprintf(str,L"%i",rcWin.top );
			SendDlgItemMessage( hWnd, IDC_JLOOKUP_WINDOW_Y, WM_SETTEXT, 0, (LPARAM)str );
				swprintf(str,L"%i",rcWin.right-rcWin.left );
			// [ w#### ] [ h#### ] [Set]:
			SendDlgItemMessage( hWnd, IDC_JLOOKUP_WINDOW_W, WM_SETTEXT, 0, (LPARAM)str );
				swprintf(str,L"%i",rcWin.bottom-rcWin.top );
			SendDlgItemMessage( hWnd, IDC_JLOOKUP_WINDOW_H, WM_SETTEXT, 0, (LPARAM)str );


			// Check the First Time button:
			if( registry.GetBOOL(".\\Misc\\FirstTime") ) {
				SendDlgItemMessage( hWnd, IDC_DICTIONARIES_INDEXING, BM_SETCHECK, BST_CHECKED, 0 );
			}

			return(TRUE);
		}
		case WM_COMMAND: {
			WORD identifier = LOWORD(wParam);
			WORD code = HIWORD(wParam);
			switch( identifier ) {
				///
				///   [x] Remember window size & position
				///
				case IDC_JLOOKUP_REMEMBERSIZE: {
					switch( code ) {
						case BN_CLICKED: {
							bool remembersize = IsDlgButtonChecked( hWnd, IDC_JLOOKUP_REMEMBERSIZE ) == BST_CHECKED;
							registry.SetBOOL(".\\Window\\RememberSize", remembersize );
							break;
						}
					}
					break;
				}
				///
				///   x [###], y [###]       [SET]
				///
				case IDC_JLOOKUP_WINDOW_X:
				case IDC_JLOOKUP_WINDOW_Y: {
					SendMessage( GetParent(hWnd), DM_SETDEFID, IDC_JLOOKUP_BUTTON_SETPOS, 0 );
					SendMessage( hWnd, DM_SETDEFID, IDC_JLOOKUP_BUTTON_SETPOS, 0 );
					break;
				}
				case IDC_JLOOKUP_BUTTON_SETPOS: {
					switch( code ) {
						case BN_CLICKED: {
							int x,y;
							Uint16 str[32];
							GetDlgItemText( hWnd, IDC_JLOOKUP_WINDOW_X, str,32-1 );
							x = _wtoi( str );
							GetDlgItemText( hWnd, IDC_JLOOKUP_WINDOW_Y, str,32-1 );
							y = _wtoi( str );

							SetWindowPos( hwnd_mainwindow, 0, x,y, 0,0, SWP_NOZORDER|SWP_NOSIZE );
							break;
						}
					}
					break;
				}
				///
				///   w [###], h [###]       [SET]
				///
				case IDC_JLOOKUP_WINDOW_W:
				case IDC_JLOOKUP_WINDOW_H: {
					SendMessage( GetParent(hWnd), DM_SETDEFID, IDC_JLOOKUP_BUTTON_SETSIZE, 0 );
					SendMessage( hWnd, DM_SETDEFID, IDC_JLOOKUP_BUTTON_SETSIZE, 0 );
					break;
				}
				case IDC_JLOOKUP_BUTTON_SETSIZE: {
					switch( code ) {
						case BN_CLICKED: {
							int w,h;
							Uint16 str[32];
							GetDlgItemText( hWnd, IDC_JLOOKUP_WINDOW_W, str,32-1 );
							w = _wtoi( str );
							GetDlgItemText( hWnd, IDC_JLOOKUP_WINDOW_H, str,32-1 );
							h = _wtoi( str );

							SetWindowPos( hwnd_mainwindow, 0, 0,0, w,h, SWP_NOZORDER|SWP_NOMOVE );
							break;
						}
					}
					break;
				}
				///
				///   [x] Pretend like this is the fist time.
				///
				case IDC_JLOOKUP_FIRSTTIME: {
					switch( code ) {
						case BN_CLICKED: {
							bool checked = IsDlgButtonChecked( hWnd, IDC_JLOOKUP_FIRSTTIME ) == BST_CHECKED;
							registry.SetBOOL(".\\Misc\\FirstTime", checked );
							break;
						}
					}
					break;
				}
			}
			return(TRUE);
		}
   }
	// Returning FALSE causes it to do the default processing for the message.
   return(FALSE); 
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateDialog_Options_JLookUp( HWND hwndParent ) {
	HWND hwnd = CreateDialog(
		g_hinst,
		MAKEINTRESOURCE( IDD_OPTIONS_JLOOKUP ), 
		hwndParent,
		(DLGPROC)DlgProc_Options_JLookUp
	);
	return( hwnd );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
