// JISToUNI2.h | by Doug Manley
///////////////////////////////////////////////////////////
//    The purpose of this file is to provide the
// ConvertEncoding() functions for converting any type of
// encoding to any other type.
///////////////////////////////////////////////////////////
#ifndef JISTOUNI2_H
#define JISTOUNI2_H
///////////////////////////////////////////////////////////
#include <GNU\iconv.h>
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
char* ConvertEncoding( char *text,int text_size, char *from,char *to ) {
	errno = 0;

	// Open the descriptor for the conversion:
	iconv_t descriptor = iconv_open( to, from );

	if( descriptor == (iconv_t)-1 ) {
		if( errno == EINVAL ) {
			TCHAR cstr[256];
				swprintf(cstr,L"conversion from '%s' to '%s' not available", from, to );
				wcscat( cstr,L"\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please." );
			MessageBox(NULL,cstr,TEXT("Error: iconv_open"), MB_OK );
		} else {
			TCHAR cstr[256];
				swprintf(cstr,L"errno: %i\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please.", errno );
			MessageBox(NULL,cstr,TEXT("Error: iconv_open"), MB_OK );
		}
		return(NULL);
	}

	char *in_buffer = text;
	char *out_buffer = new char[2*text_size+2];
		if( !out_buffer ) {
			MessageBox(NULL,L"out_buffer not created!",L"Error",MB_OK );
			return(NULL);
		}
		out_buffer[0] = '\0';

	size_t inbytes = text_size,
   		 outbytes = 2*text_size+2 - 2;
	char *in = in_buffer;
	char *out = out_buffer;

	// Clear the buffer:
	for( int i = 0; i < outbytes; i++ ) {  out_buffer[i] = 0;  }

	size_t nconv = iconv( descriptor, (const char**)&in, &inbytes,
																   &out,&outbytes
	);//*/
	// (on error, this returns the number of bytes left to convert).

	// Close the descriptor:
	iconv_close( descriptor );

	if( nconv == (size_t)0 ) {
		// Okay :)
		return( out_buffer );
	} else if( nconv == (size_t)-1 ) {
	// It's an error if there are still bytes to convert.
		if( errno == E2BIG ) {
			MessageBox(NULL,TEXT("There is not sufficient room at *outbuf\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please."),TEXT("Error: iconv"), MB_OK );
		} else if( errno == EILSEQ	) {
			MessageBox(NULL,TEXT("An invalid multibyte sequence has been encountered in the input."),TEXT("Error: iconv"), MB_OK );
		} else if( errno == EINVAL	) {
			MessageBox(NULL,TEXT("An incomplete multibyte sequence has been encountered in the input."),TEXT("Error: iconv"),MB_OK );
		} else if( errno == 0 ) {
		// errno == 0 seems to be when it just can't convert the text (not right encoding).
			// So, just make a Uint16 string out of it and return it:
			//MessageBox(NULL,L"nconv == -1\nerrno == 0",L"iconv",MB_OK );
			fwprintf(g_flogu,L"iconv: ERROR: (errno==0).  out_buffer: (%S).\n",out_buffer );
			// Just try to spit back what came in:
			/*for( int i = 0; i < strlen(text); i++ ) {
				strline += (Uint16)text[i];
			}//*/
		} else {
			TCHAR cstr[1024];
				swprintf(cstr,L"errno: %i\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please.", errno );
			MessageBox(NULL,cstr,TEXT("Error: iconv"), MB_OK );
		}
	} else {
	// The result is the number of non-reversible conversions made.
	}
	return(NULL);
}

int ConvertEncoding( char *text,int text_size, char *from,char *to, wstring &strline ) {
	char *result = ConvertEncoding( text,text_size, from,to );
	if( result ) {
		strline = (Uint16*)result;
		delete [] result;
		return(0);
	} else {
		return(-1);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
int ConvertEncoding( char *text,int text_size, char *from,char *to, string &strline ) {
	int status = 0;
	errno = 0;
	strline = "";

	// Open the descriptor for the conversion:
	iconv_t descriptor = iconv_open( to, from );

	if( descriptor == (iconv_t)-1 ) {
		if( errno == EINVAL ) {
			TCHAR cstr[256];
				swprintf(cstr,L"conversion from '%s' to '%s' not available", from, to );
				wcscat( cstr,L"\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please." );
			MessageBox(NULL,cstr,TEXT("Error: iconv_open"), MB_OK );
		} else {
			TCHAR cstr[256];
				swprintf(cstr,L"errno: %i\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please.", errno );
			MessageBox(NULL,cstr,TEXT("Error: iconv_open"), MB_OK );
		}
		strline = "(error)";
		return(-1);
	}

	char *in_buffer = text;
	char *out_buffer = new char[2*text_size+2];
		out_buffer[0] = '\0';

	size_t inbytes = text_size,
   		 outbytes = 2*text_size+2;
	char *in = in_buffer;
	char *out = out_buffer;

	for( int i = 0; i < outbytes; i++ ) {
		out_buffer[i] = 0;
	}

	size_t nconv = iconv( descriptor, (const char**)&in, &inbytes,
																   &out,&outbytes
	);//*/
	// (this returns the number of bytes left to convert).

	// Close the descriptor:
	iconv_close( descriptor );

	if( nconv == (size_t)0 ) {
		// Okay :)
		// Actually make the new string:
		char *cstr = (char*)out_buffer;
		strline = cstr;
	} else if( nconv == (size_t)-1 ) {
	// It's an error if there are still bytes to convert.
		status = -1;
		if( errno == E2BIG ) {
			MessageBox(NULL,TEXT("There is not sufficient room at *outbuf\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please."),TEXT("Error: iconv"), MB_OK );
		} else if( errno == EILSEQ	) {
			MessageBox(NULL,TEXT("An invalid multibyte sequence has been encountered in the input."),TEXT("Error: iconv"), MB_OK );
		} else if( errno == EINVAL	) {
			MessageBox(NULL,TEXT("An incomplete multibyte sequence has been encountered in the input."),TEXT("Error: iconv"),MB_OK );
		} else if( errno == 0 ) {
		// errno == 0 seems to be when it just can't convert the text (not right encoding).
			// So, just make a Uint16 string out of it and return it:
			fwprintf(g_flogu,L"iconv: ERROR: (errno==0).  out_buffer: (%S).\n",out_buffer );
			for( int i = 0; i < strlen(text); i++ ) {
				strline += text[i];
			}
		} else {
			TCHAR cstr[256];
				swprintf(cstr,L"errno: %i\n\nPlease report this error to Doug Manley (ssvegeta7@yahoo.com), please.", errno );
			MessageBox(NULL,cstr,TEXT("Error: iconv"), MB_OK );
		}
	} else {
	// The result is the number of non-reversible conversions made.
	}
	delete [] out_buffer;
	return( status );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
