// JLookUp.cpp | by Doug Manley
///////////////////////////////////////////////////////////
//
//////////////////////////////////////////////
///   #defines for internal program use:   ///
//////////////////////////////////////////////
//
/* This allows the program to print to out.conversion.txt,
	printing out its progress in converting a ROMAji string to
	a kana one. */
//#define DEBUG_CONVERSION
//
/* This is for different versions of JLookUp. */
//#define VERSION_NT
//
///////////////////////////////////////////////////////////
// We're going to be using Uint16s for text:
#define UNICODE
#include <windows.h>
#include <gdiplus.h>
using namespace Gdiplus;

#include <Commctrl.h>

#include <RichEdit.h>

#include <stdio.h>

#include <stdlib.h>
#include <string>
#include <vector>
using namespace std;
///////////////////////////////////////////////////////////
// Typedefs:
typedef unsigned char Uint8;
typedef unsigned short Uint16;
typedef unsigned long int Uint32;
///////////////////////////////////////////////////////////
// Version information:
#ifdef VERSION_NT
Uint16 version_info[] = L"1.0.6b";
Uint16 version_type[] = L"NT";
#else
Uint16 version_info[] = L"1.0.6b (non-NT version)";
Uint16 version_type[] = L"98";
#endif
///////////////////////////////////////////////////////////
// Wide-string comparison with wildcards:
#include "wc_wcsncmp.h"

// The project's resource information:
#include "resource.h"

// Virtual Key definitions:
#include "VK.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Really global things:
FILE *g_flogu;

HINSTANCE g_hinst;

// Cursors:
HCURSOR cursor_arrow,
		  cursor_resize_horizontal,
		  cursor_resize_vertical,
		  cursor_hand;

// Bitmaps:
HBITMAP bitmap_sort_asc1,
		  bitmap_sort_asc2,
		  bitmap_sort_dsc1,
		  bitmap_sort_dsc2;
///////////////////////////////////////////////////////////
HFONT hfont_msgothic12;// = CreateFont(12, 0,0,0,0,0,0,0,0,0,0,0,0,TEXT("MS Gothic") );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
enum {
	SEARCH_STYLE_START,
	SEARCH_STYLE_MIDDLE,
	SEARCH_STYLE_END,
	SEARCH_STYLE_WHOLE,
};
int search_style_location = SEARCH_STYLE_START;

// Match multiple substrings:
int search_matchsubstrings = 0;

// Force using word instead of kana.
int search_forcewordsearch = 0;

enum {
	SEARCH_CONVERSION_ROMAJI_NONE = 0,
	SEARCH_CONVERSION_ROMAJI_TOKANA = 1,
};
int search_conversion_romaji = SEARCH_CONVERSION_ROMAJI_NONE;

enum {
	SEARCH_CONVERSION_KANA_NONE = 0,
	SEARCH_CONVERSION_KANA_EQUAL = 1,
};
int search_conversion_kana = SEARCH_CONVERSION_KANA_NONE;

// Show the processed input:
int search_conversion_show = 0;

int search_results_max = 200;
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// The Main Window:
HWND hwnd_mainwindow;
	// Subwindows of the Main Window:

	// The Status Window:
	HWND hwnd_main_status;

	// The Search Window:
	HWND hwnd_searchwindow;
		// SubWindows of the Search Window:
		HWND hwnd_searchrebar;
			const int WID_SEARCH_REBAR = 1032;
			HWND hwnd_searchrebar_toolbar_location;
			HWND hwnd_searchrebar_toolbar_strings;
			HWND hwnd_searchrebar_toolbar_conversion;
			HWND hwnd_searchrebar_combobox_encode;
				const int WID_SEARCH_COMBOBOX = 1101;
			HWND hwnd_searchrebar_toolbar_encodego;
			HWND hwnd_searchrebar_edit_resultsize;
				const int WID_SEARCH_RESULTSMAX = 1103;

		HWND hwnd_searchedit;
			const int WID_SEARCH_EDIT = 1111;

	// The Kanji Window:
	HWND hwnd_kanjiwindow;
		// Subwindows of the Kanji Window:
		HWND hwnd_kanjilist1 = NULL;
			const int WID_KANJI_LIST1 = 1211;
		HWND hwnd_kanjilist2 = NULL;
			const int WID_KANJI_LIST2 = 1221;

	// The Result Window:
	HWND hwnd_resultwindow;
		// Subwindows of the Result Window:
		HWND hwnd_resultlist1 = NULL;
			const int WID_RESULT_LIST1 = 1311;
		HWND hwnd_resultlist2 = NULL;
			const int WID_RESULT_LIST2 = 1321;

	// The Radical Lookup Window:
	HWND hwnd_radicallookup;
///////////////////////////////////////////////////////////
#include "ToString.h"
///////////////////////////////////////////////////////////
void SetStatusBarText( wstring str ) {
	SendMessage( hwnd_main_status, SB_SETTEXT, 0, (LPARAM)(LPSTR)str.c_str() );
}
///////////////////////////////////////////////////////////
// For the About Window:
#include "Window_About.h"

// For the program options:
#include "Registry.h"
Registry registry;

// This makes the default options file:
#include "MakeNewOptionsFile.h"

// Helpful Header functions:
#include "Header.h"

#include "JISToUNI2.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#include "Dictionary_EDICT.h"
///////////////////////////////////////////////////////////
#include "Edict.h"
#include "Kanjidic.h"
#include "Edict_English.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// The currently selected kanji:
int kanji_selected = -1;

// This is is the currently selected entry:
int entry_selected = -1;
///////////////////////////////////////////////////////////
#include "Threads.h"
///////////////////////////////////////////////////////////
#include "Conversion.h"
#include "Search.h"

#include "Edict_Sort.h"
#include "Search_Dict.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#include "Search_Toolbar.h"
#include "Result_Window.h"
#include "Kanji_Window.h"
#include "RadicalLookup_Window.h"
#include "Search_Window.h"

#include "Main_Window.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
INT WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, PSTR, INT iCmdShow ) {
	///////////////////////////////
	///   GLOBAL OUTPUT FILES   ///
	///////////////////////////////
	g_flogu = fopen("out.uni.txt","wb");
		fwprintf(g_flogu,L"%c",0xFEFF);
	fwprintf(g_flogu,L"********************"); for( int e = 0; e < wcslen(version_info); e++ ) {  fwprintf(g_flogu,L"*");  }  fwprintf(g_flogu,L"\n");
	fwprintf(g_flogu,L"***   JLookUp %s   ***\n",version_info );
	fwprintf(g_flogu,L"********************"); for( e = 0; e < wcslen(version_info); e++ ) {  fwprintf(g_flogu,L"*");  }  fwprintf(g_flogu,L"\n");
	fwprintf(g_flogu,L"   (Unicode output file)\n");



	/////////////////////////////
	///   LOAD THE REGISTRY   ///
	/////////////////////////////
	fwprintf(g_flogu,L"Loading Options...............");
	bool loaded = registry.LoadFromFile( "Options.jlreg" );
	if( !loaded ) {
		fwprintf(g_flogu,L"FAILED\n");
		fwprintf(g_flogu,L"   Creating and using default options file...\n");
		fwprintf(g_flogu,L"Loading Options...");
		MakeNewOptionsFile("Options.jlreg");
		loaded = registry.LoadFromFile("Options.jlreg");
		if( loaded ) {  fwprintf(g_flogu,L"OKAY\n");  }
		else {  fwprintf(g_flogu,L"FAILED\n");  }
	} else {  fwprintf(g_flogu,L"OKAY\n");  }
	registry.CD( "." );

	// (for debugging):
	//registry.Print();



	////////////////////////////////////
	///   Set the global HINSTANCE   ///
	////////////////////////////////////
	g_hinst = hInstance;
   
	////////////////////////////
   ///    Initialize GDI+   ///
	////////////////////////////
	fwprintf(g_flogu,L"Intializing GDI+..............");
		GdiplusStartupInput gdiplusStartupInput;
		ULONG_PTR gdiplusToken;
   int status = GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
	if( status == Ok ) {  fwprintf(g_flogu,L"OKAY\n");  }
	else {  fwprintf(g_flogu,L"FAILED\n");  exit(0);  }

	/////////////////////////////////////////////////////////
	///   Ensure that the common control DLL is loaded.   ///
	/////////////////////////////////////////////////////////
	fwprintf(g_flogu,L"Intializing Common Controls...");
	INITCOMMONCONTROLSEX icex;
		icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
		icex.dwICC  = ICC_BAR_CLASSES | ICC_COOL_CLASSES | ICC_LISTVIEW_CLASSES;
	BOOL init = InitCommonControlsEx(&icex);
	if( init ) {  fwprintf(g_flogu,L"OKAY\n");  }
	else {  fwprintf(g_flogu,L"FAILED\n");  exit(0);  }


	//////////////////////////////
	///    Load the cursors:   ///
	//////////////////////////////
	cursor_arrow = LoadCursor( NULL,IDC_ARROW );
	cursor_resize_horizontal = LoadCursor( NULL,IDC_SIZEWE );
	cursor_resize_vertical = LoadCursor( NULL,IDC_SIZENS );
	cursor_hand = LoadCursor( NULL,IDC_HAND );

	/////////////////////////////
	///   Load the bitmaps:   ///
	/////////////////////////////
	bitmap_sort_asc1 = LoadBitmap( g_hinst, MAKEINTRESOURCE( IDB_HEADER_SORT_ASC1 ) );
	bitmap_sort_asc2 = LoadBitmap( g_hinst, MAKEINTRESOURCE( IDB_HEADER_SORT_ASC2 ) );
	bitmap_sort_dsc1 = LoadBitmap( g_hinst, MAKEINTRESOURCE( IDB_HEADER_SORT_DSC1 ) );
	bitmap_sort_dsc2 = LoadBitmap( g_hinst, MAKEINTRESOURCE( IDB_HEADER_SORT_DSC2 ) );

	

	/////////////////////////
	///   RESULT WINDOW   ///
	/////////////////////////
	// Fonts:
		hfont_result_word = CreateFont(
			-MulDiv( registry.GetINT(".\\Results\\Fonts\\Word\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Results\\Fonts\\Word\\Font") ).c_str()
		);
		hfont_result_kana = CreateFont(
			-MulDiv( registry.GetINT(".\\Results\\Fonts\\Kana\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Results\\Fonts\\Kana\\Font") ).c_str()
		);
		hfont_result_meaning = CreateFont(
			-MulDiv( registry.GetINT(".\\Results\\Fonts\\Meaning\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Results\\Fonts\\Meaning\\Font") ).c_str()
		);
		hfont_result_info = CreateFont(
			-MulDiv( registry.GetINT(".\\Results\\Fonts\\Info\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Results\\Fonts\\Info\\Font") ).c_str()
		);

	// Colors:
		color_result_word.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Fonts\\Word\\Color") );
		color_result_kana.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Fonts\\Kana\\Color") );
		color_result_meaning.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Fonts\\Meaning\\Color") );
		color_result_info.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Fonts\\Info\\Color") );

		color_result_bg1.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Colors\\BG1") );
		color_result_bg2.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Colors\\BG2") );
		color_result_border_inside.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Colors\\Border_Inside") );
		color_result_border_outside.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Colors\\Border_Outside") );
		color_result_windowbg.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Results\\Colors\\BGWindow") );

	////////////////////////
	///   KANJI WINDOW   ///
	////////////////////////
	// Fonts:
		hfont_kanji_kanji = CreateFont(
			-MulDiv( registry.GetINT(".\\Kanji\\Fonts\\Kanji\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\Kanji\\Font") ).c_str()
		);
		hfont_kanji_onkun = CreateFont(
			-MulDiv( registry.GetINT(".\\Kanji\\Fonts\\On\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\On\\Font") ).c_str()
		);
		hfont_kanji_meaning = CreateFont(
			-MulDiv( registry.GetINT(".\\Kanji\\Fonts\\Meaning\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Kanji\\Fonts\\Meaning\\Font") ).c_str()
		);

	// Colors:
		color_kanji.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Fonts\\Kanji\\Color") );
		color_kanji_on.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Fonts\\On\\Color") );
		color_kanji_kun.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Fonts\\Kun\\Color") );
		color_kanji_meaning.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Fonts\\Meaning\\Color") );

		color_kanji_bg1.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Colors\\BG1") );
		color_kanji_bg2.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Colors\\BG2") );
		color_kanji_border_inside.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Colors\\Border_Inside") );
		color_kanji_border_outside.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Colors\\Border_Outside") );
		color_kanji_windowbg.SetFromCOLORREF( (COLORREF)registry.GetINT(".\\Kanji\\Colors\\BGWindow") );


	////////////////
	///   MISC   ///
	////////////////
	// Font:
		hfont_msgothic12 = CreateFont(
			-MulDiv( registry.GetINT(".\\Misc\\Font\\Size"), GetDeviceCaps(GetDC(hwnd_resultwindow), LOGPIXELSY), 72),
			0,0,0,0,0,0,0,0,0,0,0,0,
			ToWString( registry.GetSTRING(".\\Misc\\Font\\Font") ).c_str()
		);


	//////////////////////////////////
	///   CREATE THE MAIN WINDOW   ///
	//////////////////////////////////
	int x = CW_USEDEFAULT, y = CW_USEDEFAULT, w = 800, h = 600;
	if( registry.GetBOOL(".\\Window\\RememberSize") ) {
		registry.CD(".\\Window");
		x = registry.GetINT("Window_x");
		y = registry.GetINT("Window_y");
		w = registry.GetINT("Window_w");
		h = registry.GetINT("Window_h");
		if( w < 30 || h < 30 || x < -300 || x > 3000 || y < -300 || y > 3000 ) {
			x = CW_USEDEFAULT; y = CW_USEDEFAULT; w = 800; h = 600;
			MessageBox(NULL,L"The window position settings were incorrect.\n\nThey have been restored to the defaults.",L"Warning:",MB_OK );
		}
	}
	hwnd_mainwindow = CreateMain_Window( NULL, x,y, w,h );


	/////////////////////////////
	///   CHECK THE BUTTONS   ///
	/////////////////////////////
	// Check the buttons:
	{
		// For the Search-type:
		{
			// The button that's down is 'search_style_location'.
			int type = registry.GetINT(".\\Search\\Toolbars\\Type" );
			switch( type ) {
				case SEARCH_STYLE_START: {		type = ID_SEARCH_START;		break;  }
				case SEARCH_STYLE_MIDDLE: {	type = ID_SEARCH_MIDDLE;	break;  }
				case SEARCH_STYLE_END: {		type = ID_SEARCH_END;		break;  }
				case SEARCH_STYLE_WHOLE: {		type = ID_SEARCH_WHOLE;		break;  }
			}
			SendMessage( hwnd_searchrebar_toolbar_location, TB_CHECKBUTTON, type, TRUE );
			SendMessage( hwnd_searchwindow, WM_COMMAND, type, NULL );
		}
		// For the match-substrings:
		if( registry.GetBOOL(".\\Search\\Toolbars\\Substr") ) {
			SendMessage( hwnd_searchrebar_toolbar_strings, TB_CHECKBUTTON, ID_MATCHSUBSTRINGS, TRUE );
			SendMessage( hwnd_searchrebar_toolbar_strings, WM_COMMAND, ID_MATCHSUBSTRINGS, NULL );
		}
		// For the search-only-word
		if( registry.GetBOOL(".\\Search\\Toolbars\\Word") ) {
			SendMessage( hwnd_searchrebar_toolbar_strings, TB_CHECKBUTTON, ID_FORCEWORDSEARCH, TRUE );
			SendMessage( hwnd_searchrebar_toolbar_strings, WM_COMMAND, ID_FORCEWORDSEARCH, NULL );
		}
		// For ROMAji conversion:
		if( registry.GetBOOL(".\\Search\\Toolbars\\Romaji-conversion") ) {
			SendMessage( hwnd_searchrebar_toolbar_conversion, TB_CHECKBUTTON, ID_SEARCH_CONVERSION_ROMAJI, TRUE );
			SendMessage( hwnd_searchrebar_toolbar_conversion, WM_COMMAND, ID_SEARCH_CONVERSION_ROMAJI, NULL );
		}
		// For kana-insensitive search:
		if( registry.GetBOOL(".\\Search\\Toolbars\\Kana-sensitive") ) {
			SendMessage( hwnd_searchrebar_toolbar_conversion, TB_CHECKBUTTON, ID_SEARCH_CONVERSION_KANA, TRUE );
			SendMessage( hwnd_searchrebar_toolbar_conversion, WM_COMMAND, ID_SEARCH_CONVERSION_KANA, NULL );
		}
		// Show the processed input?
		if( registry.GetBOOL(".\\Search\\Toolbars\\Show") ) {
			SendMessage( hwnd_searchrebar_toolbar_conversion, TB_CHECKBUTTON, ID_SEARCH_CONVERSION_SHOW, TRUE );
			SendMessage( hwnd_searchrebar_toolbar_conversion, WM_COMMAND, ID_SEARCH_CONVERSION_SHOW, NULL );
		}

		// For the encoding (index):
		SendMessage( hwnd_searchrebar_combobox_encode,CB_SETCURSEL, registry.GetINT(".\\Search\\Toolbars\\Encoding"),0 );
	}





	// Select the sort buttons for Results:
	// (the secondary):
	{
		NMHEADER nmh;
			nmh.iItem = registry.GetINT(".\\Results\\Sort2 Button"); // The index of the item.
			nmh.iButton = 0;
			NMHDR hdr;
				hdr.code = HDN_ITEMCLICK;
				hdr.hwndFrom = registry.GetINT(".\\Results\\Sort2 Level")==1?hwnd_resultlist1:hwnd_resultlist2;
				hdr.idFrom = 0;
			nmh.hdr = hdr;
		SendMessage( hwnd_resultwindow, WM_NOTIFY, registry.GetINT(".\\Results\\Sort2 Level")==1?WID_RESULT_LIST1:WID_RESULT_LIST2, (LPARAM)&nmh );
		if( !registry.GetBOOL(".\\Results\\Sort2 Asc") ) {
			SendMessage( hwnd_resultwindow, WM_NOTIFY, registry.GetINT(".\\Results\\Sort2 Level")==1?WID_RESULT_LIST1:WID_RESULT_LIST2, (LPARAM)&nmh );
		}
	}
	// (the primary):
	{
		NMHEADER nmh;
			nmh.iItem = registry.GetINT(".\\Results\\Sort1 Button"); // The index of the item.
			nmh.iButton = 0;
			NMHDR hdr;
				hdr.code = HDN_ITEMCLICK;
				hdr.hwndFrom = registry.GetINT(".\\Results\\Sort1 Level")==1?hwnd_resultlist1:hwnd_resultlist2;
				hdr.idFrom = 0;
			nmh.hdr = hdr;
		SendMessage( hwnd_resultwindow, WM_NOTIFY, registry.GetINT(".\\Results\\Sort1 Level")==1?WID_RESULT_LIST1:WID_RESULT_LIST2, (LPARAM)&nmh );
		if( !registry.GetBOOL(".\\Results\\Sort1 Asc") ) {
			SendMessage( hwnd_resultwindow, WM_NOTIFY, registry.GetINT(".\\Results\\Sort1 Level")==1?WID_RESULT_LIST1:WID_RESULT_LIST2, (LPARAM)&nmh );
		}
	}

	// Select the sort buttons for Kanji:
	// (the secondary):
	{
		NMHEADER nmh;
			nmh.iItem = registry.GetINT(".\\Kanji\\Sort2 Button"); // The index of the item.
			nmh.iButton = 0;
			NMHDR hdr;
				hdr.code = HDN_ITEMCLICK;
				hdr.hwndFrom = registry.GetINT(".\\Kanji\\Sort2 Level")==1?hwnd_kanjilist1:hwnd_kanjilist2;
				hdr.idFrom = 0;
			nmh.hdr = hdr;
		SendMessage( hwnd_kanjiwindow, WM_NOTIFY, registry.GetINT(".\\Kanji\\Sort2 Level")==1?WID_KANJI_LIST1:WID_KANJI_LIST2, (LPARAM)&nmh );
		if( !registry.GetBOOL(".\\Kanji\\Sort2 Asc") ) {
			SendMessage( hwnd_kanjiwindow, WM_NOTIFY, registry.GetINT(".\\Kanji\\Sort2 Level")==1?WID_KANJI_LIST1:WID_KANJI_LIST2, (LPARAM)&nmh );
		}
	}
	// (the primary):
	{
		NMHEADER nmh;
			nmh.iItem = registry.GetINT(".\\Kanji\\Sort1 Button"); // The index of the item.
			nmh.iButton = 0;
			NMHDR hdr;
				hdr.code = HDN_ITEMCLICK;
				hdr.hwndFrom = registry.GetINT(".\\Kanji\\Sort1 Level")==1?hwnd_kanjilist1:hwnd_kanjilist2;
				hdr.idFrom = 0;
			nmh.hdr = hdr;
		SendMessage( hwnd_kanjiwindow, WM_NOTIFY, registry.GetINT(".\\Kanji\\Sort1 Level")==1?WID_KANJI_LIST1:WID_KANJI_LIST2, (LPARAM)&nmh );
		if( !registry.GetBOOL(".\\Kanji\\Sort1 Asc") ) {
			SendMessage( hwnd_kanjiwindow, WM_NOTIFY, registry.GetINT(".\\Kanji\\Sort1 Level")==1?WID_KANJI_LIST1:WID_KANJI_LIST2, (LPARAM)&nmh );
		}
	}



	// Select the text for ease.
	SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
	SetFocus( hwnd_searchedit );




	// Actually show the main window:
	UpdateWindow(hwnd_mainwindow);








	if( registry.GetBOOL(".\\Misc\\FirstTime") ) {
		Uint16 text[5000]; swprintf(text,L"\
Hello!! It would appear that this is your first time using JLookUp.\n\
\n\
First of all, I cannot stess enough that you should look at 'index.html'\n\
in the Documentation folder.\n\
\n\
With that said...\n\
ABOUT SEARCHING:\n\
     To search for Japanese text, just type ROMAji or Japanese and press Enter.\n\
          Valid wildcards: _ (anything), and ? (one character).\n\
\n\
     To search for Engish text, type\n\
          EN:EnglishTextHere\n\
     where EngishTextHere is some English text, like:\n\
          to laugh\n\
     or\n\
          happy\n\
     NOTE: English act basically like normaly search-engine searches\n\
     If you want to search for words with EXACTLY the meaning that you want,\n\
     put your search in quotes DIRECTLY after the colon.  For example,\n\
          EN:\"happy\"\n\
\n\
Are you tired of this message?\n\
YES: never show it again.\n\
NO: show it next time, too.");
		int result = MessageBox(hwnd_mainwindow,text,
			L"This is the first time that JLookUp has been opened.",MB_YESNO
		);
		if( result == IDYES ) {
			registry.SetBOOL(".\\Misc\\FirstTime",FALSE);
		}
	}











	// This function has no use anymore:
	InitThreads();

	///////////////////////////////////////////////
	//   Load ALL of the EDICT related stuff:   ///
	///////////////////////////////////////////////
	Load_Thread_LoadAllEDICTS();

	{
	MSG msg;
	while( true ) {
		if( !thread_searching ) {  break;  }
		
		PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
		TranslateMessage(&msg);
		//if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  break;  }
		if( msg.message == WM_QUIT ) {  exit(0);  }
		DispatchMessage(&msg);
		Sleep( 10L );
	}
	DestroyWindow(hwnd_searching);
	RedrawWindow( hwnd_mainwindow, NULL,NULL, RDW_INVALIDATE );
	}//*/












	MSG msg;
   while( GetMessage(&msg, NULL, 0, 0) ) {
		if( msg.message == WM_QUIT ) {  break;  }
      TranslateMessage(&msg);
		if( msg.message == WM_QUIT ) {  break;  }
      DispatchMessage(&msg);
		if( msg.message == WM_QUIT ) {  break;  }
   }




	/////////////////////////////
	///   SAVE THE REGISTRY   ///
	/////////////////////////////
	fwprintf(g_flogu,L"********************************\n");
	fwprintf(g_flogu,L"***   Saving Options.jlreg   ***\n");
	fwprintf(g_flogu,L"********************************\n");
	// Set the values for the window:
	if( registry.GetBOOL(".\\Window\\RememberSize") ) {
		// Get information about the window's placement.
		// We do this to get its correct position & size even if it's minimized.
		WINDOWPLACEMENT wp;
			wp.length = sizeof(WINDOWPLACEMENT);
		GetWindowPlacement( hwnd_mainwindow, &wp );
		registry.CD(".\\Window");
		registry.SetINT("Window_x", wp.rcNormalPosition.left );
		registry.SetINT("Window_y", wp.rcNormalPosition.top );
		registry.SetINT("Window_w", (wp.rcNormalPosition.right-wp.rcNormalPosition.left) );
		registry.SetINT("Window_h", (wp.rcNormalPosition.bottom-wp.rcNormalPosition.top) );
		registry.SetINT("Div_vert", div_main.GetHalf(DIV_TOP)->GetRECT().bottom );
		registry.SetINT("Div_vert_bottom_horiz", div_main.GetHalf(DIV_BOTTOM)->GetHalf(DIV_LEFT)->GetRECT().right );
	}
	// Set the value for the result-size:
	{
		Uint16 strresults[35];
			SendMessage( hwnd_searchrebar_edit_resultsize,WM_GETTEXT,35-1,(LPARAM)strresults );
		search_results_max = _wtoi( strresults );
		registry.SetINT(".\\Results\\Size", search_results_max );
	}
	// Set the values for the buttons:
	{
		// For the Search-type:
		{
			// The button that's down is 'search_style_location'.
			int type = 0;
			switch( search_style_location ) {
				case SEARCH_STYLE_START: {		type = ID_SEARCH_START;		break;  }
				case SEARCH_STYLE_MIDDLE: {	type = ID_SEARCH_MIDDLE;	break;  }
				case SEARCH_STYLE_END: {		type = ID_SEARCH_END;		break;  }
				case SEARCH_STYLE_WHOLE: {		type = ID_SEARCH_WHOLE;		break;  }
			}
			registry.SetINT(".\\Search\\Toolbars\\Type",type );
		}
		// For the match-substrings:
		registry.SetBOOL(".\\Search\\Toolbars\\Substr",search_matchsubstrings==1 );
		// For the search-only-word
		registry.SetBOOL(".\\Search\\Toolbars\\Word",search_forcewordsearch==1 );
		// For ROMAji conversion:
		registry.SetBOOL(".\\Search\\Toolbars\\Romaji-conversion",search_conversion_romaji==1 );
		// For kana-insensitive search:
		registry.SetBOOL(".\\Search\\Toolbars\\Kana-sensitive",search_conversion_kana==1 );
		// Show the processed input?
		registry.SetBOOL(".\\Search\\Toolbars\\Show",search_conversion_show==1 );

		// For the encoding (index):
		registry.SetINT(".\\Search\\Toolbars\\Encoding", SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 ) );
	}
	// Set the sort-by's for Results:
	registry.SetINT(".\\Results\\Sort1 Level", sort_by[PRIMARY].window_id==WID_RESULT_LIST1?1:2 );
	registry.SetINT(".\\Results\\Sort1 Button", sort_by[PRIMARY].index );
	registry.SetBOOL(".\\Results\\Sort1 Asc", sort_by[PRIMARY].sortstyle==ASCENDING );

	registry.SetINT(".\\Results\\Sort2 Level", sort_by[SECONDARY].window_id==WID_RESULT_LIST1?1:2 );
	registry.SetINT(".\\Results\\Sort2 Button", sort_by[SECONDARY].index );
	registry.SetBOOL(".\\Results\\Sort2 Asc", sort_by[SECONDARY].sortstyle==ASCENDING );

	// Set the sort-by's for Kanji:
	registry.SetINT(".\\Kanji\\Sort1 Level", sort_kanji_by[PRIMARY].window_id==WID_KANJI_LIST1?1:2 );
	registry.SetINT(".\\Kanji\\Sort1 Button", sort_kanji_by[PRIMARY].index );
	registry.SetBOOL(".\\Kanji\\Sort1 Asc", sort_kanji_by[PRIMARY].sortstyle==ASCENDING );

	registry.SetINT(".\\Kanji\\Sort2 Level", sort_kanji_by[SECONDARY].window_id==WID_KANJI_LIST1?1:2 );
	registry.SetINT(".\\Kanji\\Sort2 Button", sort_kanji_by[SECONDARY].index );
	registry.SetBOOL(".\\Kanji\\Sort2 Asc", sort_kanji_by[SECONDARY].sortstyle==ASCENDING );


	// Set the percentages for the Result Window headers:
	for( int i = 0; i < 4; i++ ) {
		char path[1024];
			sprintf(path,".\\Results\\HeaderWindow1\\Header%i",i );
		HDITEM hditem;
			hditem.mask = HDI_LPARAM;
		Header_GetItem( hwnd_resultlist1, i, &hditem );
		registry.SetINT(path,(int)(double)hditem.lParam);
	}
	for( i = 0; i < 2; i++ ) {
		char path[1024];
			sprintf(path,".\\Results\\HeaderWindow2\\Header%i",i );
		HDITEM hditem;
			hditem.mask = HDI_LPARAM;
		Header_GetItem( hwnd_resultlist2, i, &hditem );
		registry.SetINT(path,(int)(double)hditem.lParam);
	}


	// Set the percentages for the Kanji Window headers:
	for( i = 0; i < 4; i++ ) {
		char path[1024];
			sprintf(path,".\\Kanji\\HeaderWindow1\\Header%i",i );
		HDITEM hditem;
			hditem.mask = HDI_LPARAM;
		Header_GetItem( hwnd_kanjilist1, i, &hditem );
		registry.SetINT(path,(int)(double)hditem.lParam);
	}
	for( i = 0; i < 2; i++ ) {
		char path[1024];
			sprintf(path,".\\Kanji\\HeaderWindow2\\Header%i",i );
		HDITEM hditem;
			hditem.mask = HDI_LPARAM;
		Header_GetItem( hwnd_kanjilist2, i, &hditem );
		registry.SetINT(path,(int)(double)hditem.lParam);
	}


	// Save all of the values to file:
	bool saved = registry.SaveToFile("Options.jlreg");
	if( saved ) {  fwprintf(g_flogu,L"Options.jlreg saved successfully.\n");  }
	else {  fwprintf(g_flogu,L"ERROR while saving Options.jlreg.\n");  }



	fwprintf(g_flogu,L"**************************\n");
	fwprintf(g_flogu,L"***   Cleaning up...   ***\n");
	fwprintf(g_flogu,L"**************************\n");
	// Clean up all of the memory used by the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		fwprintf(g_flogu,L"Deleting EDICT %i / %i...",d+1,edict_dictionaries.size() );
		DeleteEDICT( edict_dictionaries[d] );

		delete edict_dictionaries[d];
		fwprintf(g_flogu,L"OKAY\n");
	}
	fwprintf(g_flogu,L"*********************\n");
	fwprintf(g_flogu,L"***   All done!   ***\n");
	fwprintf(g_flogu,L"*********************\n");

	fclose( g_flogu );

   // Shut down GDI:
   GdiplusShutdown(gdiplusToken);
   return( msg.wParam );
}  // WinMain
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
