// Result_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef RESULT_WINDOW_H
#define RESULT_WINDOW_H
///////////////////////////////////////////////////////////
const int WM_MOUSEWHEEL = 0x020A;
const int WHEEL_DELTA = 120; // Value for rolling one dete
///////////////////////////////////////////////////////////
// These are for the drawing operations:
int bx = 3, by = 3;
int result_lineh1 = 0;
int result_lineh2 = 0;
int entry_height = 0;

// This marks where to start drawing entries:
int start_entry = 0;

// This is for maintaining the size of the client area:
RECT rcClient_result;

// These are the fonts for various entry parts:
HFONT hfont_result_word;// = CreateFont(14, 0,0,0,0,0,0,0,0,0,0,0,0,L"MS Mincho" );
HFONT hfont_result_kana;// = CreateFont(14, 0,0,0,0,0,0,0,0,0,0,0,0,L"MS Mincho" );
HFONT hfont_result_meaning;// = CreateFont(14, 0,0,0,0,0,0,0,0,0,0,0,0,L"Arial" );
HFONT hfont_result_info;

// These are the colors for various entry parts:
Color color_result_word( 0,0,96 );
Color color_result_kana( 0,0,0 );
Color color_result_info( 255,0,96 );
Color color_result_meaning( 0,0,0 );

// These are the colors for the alternating entry backgrounds:
Color color_result_bg1( 255,255,255 );
Color color_result_bg2( 196,196,196 );

// This is the color of the border around selected objects:
Color color_result_border_outside( 0,0,0 );
// This is for the border within an object:
Color color_result_border_inside( 128,128,128 );

// This is the background color of the whole window:
Color color_result_windowbg( 255,255,255 );
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#include "Result_List.h"
#include "EDICT_Sort.h"

// For copying entry info:
#include "Edict_Copy.h"

// For inverting colors:
#include "Invert.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// For the tooltip-like boxes:
#include "Result_Hover.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/* This function returns the index of the entry that
   currently contains the point (mx,my). */
int ResultWindow_GetEntryFromPoint( int mx, int my ) {
	// Get vertical scroll bar position:
	SCROLLINFO si;
		si.cbSize = sizeof(si);
		si.fMask  = SIF_POS;
	GetScrollInfo( hwnd_resultwindow, SB_VERT, &si );
	start_entry = si.nPos;

	int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
	int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

	int return_entry = -1;
	for( int e = start_entry; e <= end_entry; e++ ) {
		int i = e-start_entry;

		// If he clicked an item (x-values are irrelevant for now), select it:
		if( my >= rcClient_result.top + i*entry_height &&
			 my <= rcClient_result.top + (i+1)*entry_height-1 ) {
			return_entry = e;
			break;
		}
	}

	return( return_entry );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Result_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	static bool tracking = false;

	static POINT hover_pos;
	static int hover_entry = -1;
	static bool hover_on = false;
	static HWND hwnd_hover = NULL;

	static int last_mx = 0, last_my = 0;

	static HMENU hmenu_result_header = NULL;

	switch( message ) {
		case WM_CREATE: {
			hwnd_resultlist1 = CreateResult_List1( hWnd );
			RECT rcList1;
				GetClientRect( hwnd_resultlist1, &rcList1 );
			hwnd_resultlist2 = CreateResult_List2( hWnd, 0,rcList1.bottom );//*/

			Header_SetBitmapMargin( hwnd_resultlist1, 0 );
			Header_SetBitmapMargin( hwnd_resultlist2, 0 );

			return(0);
		}
		case WM_SIZE: {
		/* The Result Window has been resized */
			// Retrieve the dimensions of the new client area:
			int yClient = HIWORD(lParam);
			int xClient = LOWORD(lParam);

			// Get the new lineh1 and lineh2 values:
			Graphics graphics( GetDC(hWnd) );
				result_lineh1 = by +
					max(
						max(
							Font( GetDC(hWnd),hfont_result_word ).GetHeight( &graphics ),
							Font( GetDC(hWnd),hfont_result_info ).GetHeight( &graphics )
						),
						Font( GetDC(hWnd),hfont_result_meaning ).GetHeight( &graphics )
					) + by;
				result_lineh2 = by + max(
					Font( GetDC(hWnd),hfont_result_kana ).GetHeight( &graphics ),
					Font( GetDC(hWnd),hfont_result_meaning ).GetHeight( &graphics )
					) + by;
			entry_height = result_lineh1 + result_lineh2;

			// Fix the windows:
			RECT rcWindow;
				GetClientRect( hWnd, &rcWindow );

			// If the window is smaller than a certain size, leave it that way and just cover it up:
			if( rcWindow.right < 100 ) {  break;  }

			// Otherwise, scale the headers:
			RECT rcList1;
				GetClientRect( hwnd_resultlist1, &rcList1 );
			Header_SetSizeScale( hwnd_resultlist1, rcWindow.right,rcList1.bottom );
			
			RECT rcList2;
				GetClientRect( hwnd_resultlist2, &rcList2 );
			Header_SetSizeScale( hwnd_resultlist2, rcWindow.right,rcList2.bottom );


			// Set the client information:
			rcClient_result.left = 0;
			rcClient_result.top = 0 + rcList1.bottom + rcList1.bottom;

			rcClient_result.right = xClient;
			rcClient_result.bottom = yClient;

			//if( entry_height == 0 ) {  MessageBox(hWnd,L"WTF",L"!KJ#L:K",MB_OK );  }
 
			// Set the vertical scrolling range and page size:
			SCROLLINFO si;
				si.cbSize = sizeof(si); 
				si.fMask  = SIF_RANGE | SIF_PAGE;// | SIF_DISABLENOSCROLL; 
				si.nMin   = 0; 
				si.nMax   = edict_results.size() - 1;
				si.nPage  = (rcClient_result.bottom-rcClient_result.top) / (entry_height); 
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE);

			// Recalculate the start based on the new size:
				si.fMask  = SIF_ALL;
         GetScrollInfo( hWnd, SB_VERT, &si );
				start_entry = si.nPos;

			// This makes resizing smooth:
			SendMessage( hWnd, WM_PAINT, 0,0 );
			return( 0 );
		}
		case WM_NOTIFY: {
			if( wParam == WID_RESULT_LIST1 || wParam == WID_RESULT_LIST2 ) {
				NMHEADER *header = (NMHEADER*)lParam;
				//fprintf(g_flog,"WM_NOTIFY: WID_RESULT_LIST1: %i (%i)\n",header->hdr.code,HDN_TRACK);
				switch( header->hdr.code ) {
					/////////////////////////////////////
					///                               ///
					///      HEADER BAR MESSAGES      ///
					///                               ///
					/////////////////////////////////////
					case HDN_TRACK: {
						int width = Header_GetWidth( header->hdr.hwndFrom );

						// Here's the original item:
						HDITEM item_original;
							item_original.mask = HDI_LPARAM|HDI_WIDTH;
						Header_GetItem( header->hdr.hwndFrom, header->iItem, &item_original );
		
						// Fix up that item:
						HDITEM *item = header->pitem;
							item->mask = HDI_LPARAM|HDI_WIDTH;
							item->lParam = (LPARAM)(double)(double(item->cxy)/double(width)*100.0);
							item->cxy = (int)(double(item->lParam)*double(width)/100.0);

						// Now, fix up the item to its right:
						int right = Header_GetItemToRight( header->hdr.hwndFrom, header->iItem );
						if( right == -1 ) {
							MessageBox(hWnd,L"Right is -1",L"Error",MB_OK );
							return(TRUE);
						}

						// If one's invisible, 'right' is the one after that one:
						while( right != -1 ) {
							// This is the one to the right:
							HDITEM itemNext;
								itemNext.mask = HDI_LPARAM|HDI_WIDTH;
							Header_GetItem( header->hdr.hwndFrom, right, &itemNext );

							if( itemNext.lParam == (LPARAM)(double)0 ) {
								// Keep going...
								right = Header_GetItemToRight( header->hdr.hwndFrom, right );
							} else {
								break;
							}
						}
						if( right == -1 ) {  return(TRUE);  }

						// This is the one to the right:
						HDITEM item2;
							item2.mask = HDI_LPARAM|HDI_WIDTH;
						Header_GetItem( header->hdr.hwndFrom, right, &item2 );
							item2.cxy = item2.cxy - (item->cxy-item_original.cxy);
							item2.lParam = (LPARAM)(double)(double(item2.cxy)/double(width)*100.0);

						if( item2.cxy > 10 && item->cxy > 10 ) {
							if( item2.cxy != item->cxy ) {
								Header_SetItem( header->hdr.hwndFrom, right, &item2 );
								Header_SetItem( header->hdr.hwndFrom, header->iItem, item );

								//RECT rc; GetClientRect(hWnd,&rc);
								//SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom) );

								RedrawWindow( header->hdr.hwndFrom, NULL, NULL, RDW_INVALIDATE );
								RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );

								fwprintf(g_flogu,L"LPARAM: %.2f\n",(double)item2.lParam );
							}
							return(FALSE); // keep going...
						} else {
							RedrawWindow( header->hdr.hwndFrom, NULL, NULL, RDW_INVALIDATE );
							RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
							return(TRUE);
						}
						break;
					}//*/
					case HDN_BEGINTRACK: {
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );

						// If it's on the far right, do NOT let it resize.
						if( Header_GetItemToRight( header->hdr.hwndFrom, header->iItem ) == -1 ) {
							return(TRUE);
						}

						// Note: Just returning TRUE dis-allows the user to drag the dividers.
						return(FALSE);
						break;
					}//*/
					case HDN_ENDTRACK: {
						RECT rc; GetClientRect(hWnd,&rc);
						SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom) );

						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/
					// This one definitely is good; don't change it.
					case HDN_ENDDRAG: {
						// Repaint the screen when a heading is moved.
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/
					case NM_RCLICK: {
						// Make the menu:
						hmenu_result_header = CreatePopupMenu();

						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 0, L"(Right click to add/remove...)" );
						AppendMenu( hmenu_result_header, MF_SEPARATOR, NULL,NULL );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 1, L"Word" );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 2, L"Type" );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 3, L"Length (Word)" );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 4, L"Length (Kana)" );
						AppendMenu( hmenu_result_header, MF_SEPARATOR, NULL,NULL );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 5, L"Kana" );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 6, L"Meaning" );
						AppendMenu( hmenu_result_header, MF_SEPARATOR, NULL,NULL );
						AppendMenu( hmenu_result_header, MF_STRING|MF_DISABLED, 7, L"(Right-click: Fix them up)" );

						for( int i = 0; i < 6; i++ ) {
							HWND hwndheader = (i<4)?hwnd_resultlist1:hwnd_resultlist2;
							int item = i;
							if( i >= 4 ) {  item -= 4;  }
							HDITEM hditem;
								hditem.mask = HDI_LPARAM;
							Header_GetItem( hwndheader, item, &hditem );
							if( hditem.lParam != (LPARAM)(double)0 ) {
								CheckMenuItem( hmenu_result_header, i+1, MF_BYCOMMAND|MF_CHECKED );
							}
						}


						POINT pt;  GetCursorPos( &pt );
						TrackPopupMenuEx( hmenu_result_header, TPM_LEFTALIGN|TPM_LEFTBUTTON, pt.x,pt.y, hWnd, NULL );
						DestroyMenu( hmenu_result_header );
							hmenu_result_header = NULL;

						return(TRUE);
						break;
					}//*/
					/*case HDN_ITEMCHANGED: {
						//RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						break;
					}//*/

					case HDN_ITEMCLICK: {
					/* When an item is clicked, sort by that item.
					   The old PRIMARY becomes the new SECONDARY. */
						//fprintf(g_flog,"WM_NOTIFY: HDN_ITEMCLICK: iItem: %i\n",header->iItem );

						// Do not allow sorting by 'Type':
						if( wParam == WID_RESULT_LIST1 && header->iItem == 1 ) {
							break;
						}

						// If he clicked the same item twice in a row:
						if( sort_by[PRIMARY].window_id == wParam && sort_by[PRIMARY].index == header->iItem ) {
							// Toggle it so that it sorts the other way:
							sort_by[PRIMARY].sortstyle ^= 1;

							// Give it an icon: (primary)
							HDITEM item;
								item.mask = HDI_BITMAP | HDI_FORMAT;
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );
						} else {
							// Take away the sort arrow on the oldest one:
							if( sort_by[SECONDARY].window_id != 0 ) {
								//fprintf(g_flog,"Want to change: %i: %i\n",sort_by[SECONDARY].window_id, sort_by[SECONDARY].index );
								HDITEM item;
									item.mask = HDI_BITMAP | HDI_FORMAT;
								HWND hwndold = sort_by[SECONDARY].window_id==WID_RESULT_LIST1?hwnd_resultlist1:hwnd_resultlist2;
								Header_GetItem( hwndold, sort_by[SECONDARY].index, &item );
									item.mask = HDI_BITMAP | HDI_FORMAT;
									item.hbm = NULL;
									item.fmt ^= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
								Header_SetItem( hwndold, sort_by[SECONDARY].index, &item );
							}

							// The secondary sort is the old primary sort:
							sort_by[SECONDARY].window_id = sort_by[PRIMARY].window_id;
							sort_by[SECONDARY].index = sort_by[PRIMARY].index;
							sort_by[SECONDARY].sortstyle = sort_by[PRIMARY].sortstyle;

							// Set the new info:
							sort_by[PRIMARY].window_id = wParam;
							sort_by[PRIMARY].index = header->iItem;
							sort_by[PRIMARY].sortstyle = ASCENDING;

							// Give it an icon: (primary)
							HDITEM item;
								item.mask = HDI_BITMAP | HDI_FORMAT;
							Header_GetItem( header->hdr.hwndFrom, header->iItem, &item );
								item.mask = HDI_BITMAP | HDI_FORMAT;
								item.hbm = (sort_by[PRIMARY].sortstyle==ASCENDING)?bitmap_sort_asc1:bitmap_sort_dsc1;
								item.fmt |= HDF_BITMAP|HDF_BITMAP_ON_RIGHT;
							Header_SetItem( header->hdr.hwndFrom, header->iItem, &item );

							// Give the secondary a different icon:
							//HDITEM item;
								item.mask = HDI_BITMAP;
								HWND hwndsecondary = sort_by[SECONDARY].window_id==WID_RESULT_LIST1?hwnd_resultlist1:hwnd_resultlist2;
							Header_GetItem( hwndsecondary, sort_by[SECONDARY].index, &item );
								item.mask = HDI_BITMAP;
								item.hbm = (sort_by[SECONDARY].sortstyle==ASCENDING)?bitmap_sort_asc2:bitmap_sort_dsc2;
							Header_SetItem( hwndsecondary, sort_by[SECONDARY].index, &item );
						}
						// If there are any results, sort them:
						SortResults(); // (this checks to see if there are any).
						break;
					}
				}
			}// if wparam == WID_RESULT_LIST1 or ~2
			return(0);
		}
		case WM_VSCROLL: {
         // Get all the vertial scroll bar information
			SCROLLINFO si; 
				si.cbSize = sizeof(si);
				si.fMask  = SIF_ALL;
         GetScrollInfo (hWnd, SB_VERT, &si);
         // Save the position for comparison later on
         start_entry = si.nPos;
         switch( LOWORD(wParam) ) {
				// user clicked the HOME keyboard key
				case SB_TOP: {  si.nPos = si.nMin;  break;  }  
				// user clicked the END keyboard key
				case SB_BOTTOM: {  si.nPos = si.nMax;  break;  }
				// user clicked the top arrow
				case SB_LINEUP: {  si.nPos -= 1;  break;  }
				// user clicked the bottom arrow
				case SB_LINEDOWN: {  si.nPos += 1;  break;  }  
				// user clicked the shaft above the scroll box
				case SB_PAGEUP: {  si.nPos -= si.nPage;  break;  }  
				// user clicked the shaft below the scroll box
				case SB_PAGEDOWN: {  si.nPos += si.nPage;  break;  }  
				// user dragged the scroll box
				case SB_THUMBTRACK: {  si.nPos = si.nTrackPos;  break;  }
         }
         // Set the position and then retrieve it.  Due to adjustments
         //   by Windows it may not be the same as the value set.
				si.fMask = SIF_POS;
         SetScrollInfo( hWnd, SB_VERT, &si, TRUE );
         GetScrollInfo( hWnd, SB_VERT, &si );
         // If the position has changed, scroll window and update it
         if( si.nPos != start_entry ) {
				RECT rc;
					GetClientRect( hWnd,&rc );
				RedrawWindow( hWnd, &rc, NULL, RDW_INVALIDATE );
				// RDW_ERASE makes it redraw the BG.
				// RDW_INVALIDATE makes it WM_PAINT.
         }
			return(0);
		}
		case WM_PAINT: {
			// If the headers aren't there, then we have nothing to do:
			if( hwnd_resultlist1 == NULL || hwnd_resultlist2 == NULL ) {  break;  }
				//fprintf(g_flog,"ResultWindow: WM_PAINT:\n");

			// If there are no results, then just clear it:
			if( edict_results.size() < 1 || thread_searching ) {
				HDC hdc;
				PAINTSTRUCT ps;
				hdc = BeginPaint(hWnd, &ps);
				   Graphics graphics(hdc);

					SolidBrush brush_result_bg( color_result_windowbg );

					RectF rcFill;
						rcFill.X = rcClient_result.left;
						rcFill.Y = rcClient_result.top;
						rcFill.Width = rcClient_result.right;
						rcFill.Height = rcClient_result.bottom;
					graphics.FillRectangle( &brush_result_bg, rcFill );

				EndPaint(hWnd, &ps);
				return(0); // An application should return 0 if it processes WM_PAINT.
			}
		
			// Now, we know that there are entries; it's time to draw them:
			HDC hdc;
			PAINTSTRUCT ps;
	      hdc = BeginPaint(hWnd, &ps);
				Graphics graphics(hdc);

				// Set up the fonts for the text:
				Font font_word( hdc,hfont_result_word );
				Font font_kana( hdc,hfont_result_kana );
				Font font_meaning( hdc,hfont_result_meaning );
				Font font_info( hdc,hfont_result_info );

				// This is for the background of the window:
				SolidBrush brush_windowbg( color_result_windowbg );

				StringFormat format(StringFormatFlagsNoWrap);
					format.SetTrimming( StringTrimmingNone );


				// Get the RECTs for drawing the items:
				RECT rcWord;
					Header_GetItemRect( hwnd_resultlist1, 0, &rcWord );
				RECT rcType;
					Header_GetItemRect( hwnd_resultlist1, 1, &rcType );
				RECT rcLengthW;
					Header_GetItemRect( hwnd_resultlist1, 2, &rcLengthW );
				RECT rcLengthK;
					Header_GetItemRect( hwnd_resultlist1, 3, &rcLengthK );

				RECT rcKana;
					Header_GetItemRect( hwnd_resultlist2, 0, &rcKana );
				RECT rcMeaning;
					Header_GetItemRect( hwnd_resultlist2, 1, &rcMeaning );


				// Get vertical scroll bar position
				SCROLLINFO si;
				   si.cbSize = sizeof(si);
		         si.fMask  = SIF_POS;
		      GetScrollInfo( hWnd, SB_VERT, &si );
			   start_entry = si.nPos;
				
				//if( entry_height == 0 ) {  MessageBox(hWnd,L"entry_height==0",L"K!J#",MB_OK );  }

				int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
				int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

				for( int e = start_entry; e <= end_entry; e++ ) {
					int i = e-start_entry;

					EDICT_Entry entry = edict_results[e];


					SolidBrush brush_bg( color_result_bg1 );
					if( e%2 ) {	brush_bg.SetColor( color_result_bg2 );	}
					SolidBrush brush_word( color_result_word );
					SolidBrush brush_kana( color_result_kana );
					SolidBrush brush_info( color_result_info );
					SolidBrush brush_meaning( color_result_meaning );
					Pen pen_border( color_result_border_outside );
					Pen pen_border_inside( color_result_border_inside );
					if( entry_selected == e ) {
						InvertColor( brush_bg );
						InvertColor( brush_word );
						InvertColor( brush_kana );
						InvertColor( brush_info );
						InvertColor( brush_meaning );
						InvertColor( pen_border );
						InvertColor( pen_border_inside );
					}



					// Fill in the rectangle that'll be the background of the entry:
					RectF rc;
						rc.X = 0;
						rc.Y = rcClient_result.top + i*entry_height;
						rc.Width = rcClient_result.right-1;
						rc.Height = entry_height-1;
					graphics.FillRectangle( &brush_bg, rc );
					

					// WORD:
					// Box:
						rc.X = rcWord.left;
						rc.Y = rcClient_result.top + i*entry_height;
						rc.Width = rcWord.right - rcWord.left - 1;
						rc.Height = result_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( entry.word.c_str(), -1, &font_word, rc, &format, &brush_word );

					// INFO:
					wstring type;
					for( int t = 0; t < entry.info.size(); t++ ) {
						type += entry.info[t];
						if( t < entry.info.size()-1 ) {
							type += L" | ";
						}
					}
					// Box:
						rc.X = rcType.left;
						rc.Y = rcClient_result.top + i * entry_height;
						rc.Width = rcType.right - rcType.left - 1;
						rc.Height = result_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( type.c_str(), -1, &font_info, rc, &format, &brush_info );

					// LENGTHW:
					// Box:
						rc.X = rcLengthW.left;
						rc.Y = rcClient_result.top + i*entry_height;
						rc.Width = rcLengthW.right - rcLengthW.left - 1;
						rc.Height = result_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					wchar_t lengthw[10]; _itow(entry.word.size(),lengthw,10);
					graphics.DrawString( wstring(lengthw).c_str(), -1, &font_meaning, rc, &format, &brush_meaning );

					// LENGTHK:
					// Box:
						rc.X = rcLengthK.left;
						rc.Y = rcClient_result.top + i*entry_height;
						rc.Width = rcLengthK.right - rcLengthK.left - 1;
						rc.Height = result_lineh1;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					wchar_t lengthk[10]; _itow(entry.kana.size(),lengthk,10);
					graphics.DrawString( wstring(lengthk).c_str(), -1, &font_meaning, rc, &format, &brush_meaning );


					// KANA:
					// Box:
						rc.X = rcKana.left;
						rc.Y = rcClient_result.top + i*entry_height + result_lineh1;
						rc.Width = rcKana.right - rcKana.left - 1;
						rc.Height = result_lineh2;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( entry.kana.c_str(), -1, &font_kana, rc, &format, &brush_kana );

					// MEANING:
					wstring meaning = Edict_GetMeaning( entry );
					// Box:
						rc.X = rcMeaning.left;
						rc.Y = rcClient_result.top + i * entry_height + result_lineh1;
						rc.Width = rcMeaning.right - rcMeaning.left - 1;
						rc.Height = result_lineh2;
					graphics.DrawRectangle( &pen_border_inside, rc );
					// Text:
						rc.X += bx;	rc.Width -= bx;
						rc.Y += by;	rc.Height -= by;
					graphics.DrawString( meaning.c_str(), -1, &font_meaning, rc, &format, &brush_meaning );



					// Draw a black bounding rectangle around everything:
						rc.X = 0;
						rc.Y = rcClient_result.top + i * entry_height;
						rc.Width = rcClient_result.right-1;
						rc.Height = entry_height-1;
					graphics.DrawRectangle( &pen_border, rc );
				}// (the for loop from start_entry to end_entry).


				// Fill in the remaining space:
				RectF rcFill;
					rcFill.X = rcClient_result.left;
					rcFill.Y = rcClient_result.top + (end_entry-start_entry+1)*entry_height;
					rcFill.Width = rcClient_result.right;
					rcFill.Height = rcClient_result.top + rcClient_result.bottom - (int)rcFill.Y;
				if( rcFill.Height > 0 ) {
					graphics.FillRectangle( &brush_windowbg, rcFill );
				}

		   EndPaint(hWnd, &ps);
			return(0); // An application should return 0 if it processes WM_PAINT.
		}//*/
		case WM_ERASEBKGND: {
			// An application should return 0 if it did NOT erase the backgound.
			// Here, we're faking that we did, so...
			return(TRUE);
		}
		case WM_MENURBUTTONUP: {
			/*	wParam
				Specifies the position of the item when the mouse was released. 
			lParam
				Handle to the menu containing the item.//*/
			if( lParam == (LPARAM)hmenu_result_header ) {
				// The click was for the Header menu:
				int id = GetMenuItemID( hmenu_result_header, wParam );
				if( id >= 1 && id <= 6 ) {
					enum { HEADER_ADD, HEADER_REMOVE };
					int action;
					int state = GetMenuState( hmenu_result_header, wParam, MF_BYPOSITION );


					int item = id-1;
					// This is the header window to use:
					HWND header = (item<4)?hwnd_resultlist1:hwnd_resultlist2;

					if( state&MF_CHECKED ) {
					////////////////////////////////////////////
					///   WE ARE GOING TO UNCHECK THE ITEM   ///
					////////////////////////////////////////////
						// If it's the only one, then stop:
						bool onlyone = true;
						int max = (item<4)?4:2;
						int item_x = item;  if( item_x >= 4 ) {  item_x -= 4;  }
						for( int i = 0; i < max; i++ ) {
							HDITEM hditem;
								hditem.mask = HDI_LPARAM;
							Header_GetItem( header, i, &hditem );
							if( hditem.lParam != (LPARAM)(double)0 && i != item_x ) {
								onlyone = false;
								break;
							}
						}
						if( onlyone ) {  return(0);  }

						// Otherwise, remove the check mark:
						CheckMenuItem( hmenu_result_header, wParam, MF_BYPOSITION );
						action = HEADER_REMOVE;
					} else {
					//////////////////////////////////////////
					///   WE ARE GOING TO CHECK THE ITEM   ///
					//////////////////////////////////////////
						// Add the check mark:
						CheckMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_CHECKED );
						action = HEADER_ADD;
					}
					// Force it to redraw the menu:
					EnableMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_GRAYED );
					EnableMenuItem( hmenu_result_header, wParam, MF_BYPOSITION|MF_ENABLED );

					// This is the array index of the item:
					if( item >= 4 ) {  item -= 4;  }
					// Here's the guy we're primarily changing:
					HDITEM hditem;
						hditem.mask = HDI_LPARAM;
					Header_GetItem( header, item, &hditem );
					// Here's the other guy who needs to pick up the slack:
					int next = -1;
					HDITEM hditem2;
						hditem2.mask = HDI_LPARAM;
					// Let it go?
					bool allow = true;
					// Okay, gogogo:
					if( action == HEADER_ADD ) {
					///////////////////////////////////
					///   ATTEMPT TO ADD THE ITEM   ///
					///////////////////////////////////
						hditem.lParam = (LPARAM)(double)10;

						// Shrink another one:
						next = Header_GetClosestItemWithlParamGreaterThan( header, item, (LPARAM)(double)10 );
						if( next == -1 ) {
							allow = false;
						} else {
							Header_GetItem( header, next, &hditem2 );
								hditem2.lParam -= (LPARAM)(double)hditem.lParam;
						}
					} else {
					//////////////////////////////////////
					///   ATTEMPT TO DELETE THE ITEM   ///
					//////////////////////////////////////

						// Add to another one:
						next = Header_GetClosestItemWithlParamGreaterThan( header, item, (LPARAM)(double)0 );
						if( next == -1 ) {
							allow = false;
						} else {
							Header_GetItem( header, next, &hditem2 );
								hditem2.lParam += (LPARAM)(double)hditem.lParam;
						}

						hditem.lParam = (LPARAM)(double)0;
					}
					// If it's possible, add the item and change another:
					if( allow ) {
						Header_SetItem( header, item, &hditem );
						Header_SetItem( header, next, &hditem2 );

						RECT rc; GetClientRect( hWnd, &rc );
						SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					}
				} else if( id == 7 ) {
					// Normalize all of the header bars:
					for( int i = 0; i < 4; i++ ) {
						HDITEM hditem;
							hditem.mask = HDI_LPARAM;
							if( i == 0 ) {      hditem.lParam = 25;  }
							else if( i== 1 ) {  hditem.lParam = 61;  }
							else if( i== 2 ) {  hditem.lParam = 7;  }
							else if( i== 3 ) {  hditem.lParam = 7;  }
						Header_SetItem( hwnd_resultlist1, i, &hditem );
					}
					for( i = 0; i < 2; i++ ) {
						HDITEM hditem;
							hditem.mask = HDI_LPARAM;
							if( i == 0 ) {      hditem.lParam = 25;  }
							else if( i== 1 ) {  hditem.lParam = 75;  }
						Header_SetItem( hwnd_resultlist2, i, &hditem );
					}
					
					RECT rc; GetClientRect( hWnd, &rc );
					SendMessage(hWnd,WM_SIZE,0,MAKELPARAM(rc.right,rc.bottom));
					RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
				}
			}//*/
			return(0);
		}
		case WM_LBUTTONDOWN: {
		/* If he clicked on an item, select that item.
		   If he clicked on blank space, deselect any selected item. */
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// Regardless of anything, this window now has keyboard focus:
			SetFocus( hWnd );

			// Select the entry that he clicked on:
			// If there are no results, do nothing:
			if( edict_results.size() == 0 ) {  break;  }

			int old_entry_selected = entry_selected;
			entry_selected = ResultWindow_GetEntryFromPoint( mx,my );

			if( old_entry_selected != entry_selected ) {
				RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
			}
			return(0); // An application should return 0 if it processes WM_LBUTTONDOWN.
		}
		case WM_LBUTTONDBLCLK: {
			// wParam :: what virtual keys are down.
			// lParam :: LONG( x, y ).
			SendMessage( hWnd,WM_LBUTTONDOWN,wParam,lParam );
			SendMessage( hWnd,WM_KEYDOWN,VK_RETURN,0 );
			return(0); // An application should return 0 if it processes WM_LBUTTONDBLCLK.
		}
		case WM_RBUTTONDOWN: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// If there's already a search going on, stop.
			if( thread_searching ) {  return(0);  }

			// Select the item that it's over:
			if( edict_results.size() < 1 ) {  return(0);  }
			// Select it:
			SendMessage( hWnd, WM_LBUTTONDOWN, wParam,lParam );

			RECT rcWindow;
				GetWindowRect(hWnd,&rcWindow);

			// Make the menu:
			HMENU hMenu = CreatePopupMenu();

			HMENU hmenu_search = CreatePopupMenu();
				AppendMenu( hmenu_search, MF_STRING, 1, L"The kanji in this word" );
				AppendMenu( hmenu_search, MF_STRING, 2, L"Words starting with this word" );
				AppendMenu( hmenu_search, MF_STRING, 3, L"Words ending with this word" );
				AppendMenu( hmenu_search, MF_STRING, 4, L"Words with this word anywhere" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_search, L"Search..." );
				AppendMenu( hMenu, MF_SEPARATOR, NULL,NULL );
			HMENU hmenu_copy = CreatePopupMenu();
				AppendMenu( hmenu_copy, MF_STRING, 5, L"This word" );
				AppendMenu( hmenu_copy, MF_STRING, 6, L"This word's kana" );
				AppendMenu( hmenu_copy, MF_STRING, 7, L"This word's meanings" );
				AppendMenu( hmenu_copy, MF_STRING, 8, L"All info on this word" );
			AppendMenu( hMenu, MF_POPUP, (UINT_PTR)hmenu_copy, L"Copy..." );

			int result = TrackPopupMenuEx( hMenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );
			if( result == 1 ) {
			// Show the kanji in this word:
				kanjidic_results.clear();
				KanjiWindow_SearchForKanjiIn( edict_results[entry_selected].word );
				KanjiWindow_SetUpForNewResults();
			} else if( result >= 2 && result <= 4 ) {
			// Search for the word in...:
				wstring str;
				if( result == 2 ) {
					str = edict_results[entry_selected].word;
					str += L"_";
				} else if( result == 3 ) {
					str = L"_";
					str += edict_results[entry_selected].word;
				} else if( result == 4 ) {
					str = L"_";
					str += edict_results[entry_selected].word;
					str += L"_";
				}
				// Set the text of the search edit control to the kanji:
				SETTEXTEX st;
					st.flags = ST_KEEPUNDO;
					st.codepage = 1200; // 1200 == unicode.
				SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)str.c_str() );

				// Select the text:
				SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );

				// Do a search with that text:
				SendMessage( hwnd_searchwindow,WM_COMMAND, ID_SEARCH_GO,0 );
			} else if( result == 5 ) {
			// Copy the word to the clipboard:
				CopySelectedEdictInfo( EINFO_WORD );
			} else if( result == 6 ) {
			// Copy the word's kana to the clipboard:
				CopySelectedEdictInfo( EINFO_KANA );
			} else if( result == 7 ) {
			// Copy the word's meanings to the clipboard:
				CopySelectedEdictInfo( EINFO_MEANING );
			} else if( result == 8 ) {
			// Copy all info on the word to the clipboard:
				CopySelectedEdictInfo( EINFO_ALL );
			}
				DestroyMenu( hmenu_search );
				DestroyMenu( hmenu_copy );
			DestroyMenu( hMenu );

			return(0); // An application should return 0 if it processes WM_RBUTTONDOWN.
		}
		case WM_MOUSEMOVE: {
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);
			last_mx = mx; last_my = my;

			// Set the cursor to the standard arrow:
			SetCursor( cursor_arrow );

			// If we have captured the mouse (and it moves out of bounds),
			// then give it back so we can get the WM_MOUSELEAVE message.
			if( hover_on ) {
				if( mx < rcClient_result.left || my < rcClient_result.top ||
					 mx >= rcClient_result.right || my >= rcClient_result.bottom ) {
					ReleaseCapture();
				}
			}

			// If the mouse moves over the window, capture it:
			HWND hwnd_focus = GetFocus();
			if( GetForegroundWindow()==hwnd_mainwindow && hwnd_focus != hWnd && !registry.GetBOOL(".\\Search\\AlwaysSearchFocus") ) {
				HWND hwnd_old = SetFocus( hWnd );
				if( hwnd_old == NULL ) {
					fwprintf(g_flogu,L"ResultWindow: WM_MOUSEMOVE:\n   SetFocus(): ERROR: %u\n",GetLastError() );
				}
			}

			// When the mouse hovers over an entry, show a pop-up view of it.

			// If there are no results, then we're not doing any pop-ups.
			if( edict_results.size() == 0 ) {  return(0);  }
			if( thread_searching ) {  return(0);  }

			// This is the entry that it's over *now*.
			int hover_entry_test = ResultWindow_GetEntryFromPoint(mx,my);
			
			// If there's already a pop-up window, and the one that it's
			// over is *different* from that one, then destroy the old one.
			if( hover_on ) {
				if( hover_entry_test != hover_entry ) {
					DestroyWindow( hwnd_hover );
					hover_on = false;
					ReleaseCapture();
				}
			}

			// Now, see if the mouse hovers over an entry.
			if( hover_entry_test != -1 ) {
				// Cancel any others:
				{
					TRACKMOUSEEVENT event;
						event.cbSize = sizeof(TRACKMOUSEEVENT);
						event.dwFlags = TME_CANCEL|TME_HOVER;
						event.dwHoverTime = 700; // hover-time (milliseconds).
						event.hwndTrack = hWnd;
					_TrackMouseEvent( &event );
				}
				TRACKMOUSEEVENT event;
					event.cbSize = sizeof(TRACKMOUSEEVENT);
					event.dwFlags = TME_HOVER;
					event.dwHoverTime = 700; // hover-time (milliseconds).
					event.hwndTrack = hWnd;
				_TrackMouseEvent( &event );
				// Set the hover variables:
				hover_entry = hover_entry_test;
				hover_pos.x = mx;
				hover_pos.y = my;
			}
			return(0); // An application should return 0 if it processes WM_MOUSEMOVE.
		}
		case WM_MOUSEHOVER: {
			int mx = LOWORD(lParam), my = HIWORD(lParam);

			// NEGATIVE CASES:
			// * If there's already a hover window up, don't draw it again:
			if( hover_on ) {  return(0);  }
			// If the mouse moved since we sent the hover message, NO:
			if( abs( hover_pos.x - mx ) > 10 || abs( hover_pos.y - my ) > 10 ) {
				return(0);
			}
			// If it trailed off into another entry, NO:
			if( ResultWindow_GetEntryFromPoint(mx,my) != hover_entry ) {
				return(0);
			}

			/////////////////////////////////
			///   MAKE THE HOVER WINDOW   ///
			/////////////////////////////////
			// Otherwise, we have a hover window to create:
			hwnd_hover = CreateResultHoverWindow( hover_entry );

			// Capture the mouse so that this window keeps getting mouse input.
			SetCapture( hWnd );

			// Set its position:
			// First, figure out where it is (this uses the parent window for positioning):
			RECT rcWin;
				GetWindowRect( hWnd, &rcWin );
			int newx = rcWin.left+mx, newy = rcWin.top+my;
			// Then, figure out how big it is:
				GetWindowRect( hwnd_hover, &rcWin );
			int neww = rcWin.right-rcWin.left, newh = rcWin.bottom-rcWin.top;

			// Get the area that a full-screen window can use:
			RECT rcDesk;
			SystemParametersInfo( SPI_GETWORKAREA, 0, &rcDesk, 0 );

			// If the pop-up goes too far off, put it back on:
			if( newx+neww >= rcDesk.right ) {  newx = rcDesk.right-neww;  }
			if( newx < 0 ) {  newx = 0;  }

			if( newy+newh >= rcDesk.bottom ) {  newy = rcDesk.bottom-newh;  }
			if( newy < 0 ) {  newy = 0;  }

			// Now, set the position and show it:
			SetWindowPos( hwnd_hover, NULL,
				newx, newy,
				0,0,
				SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW
			);

			// Yes, we have a hover window up.
			hover_on = true;

			// We want to know if the mouse leaves the window, so that we
			// can destroy the hover window then.
			TRACKMOUSEEVENT event;
				event.cbSize = sizeof(TRACKMOUSEEVENT);
				event.dwFlags = TME_LEAVE;
				event.hwndTrack = hWnd;
			_TrackMouseEvent( &event );
			return(0);
		}
		case WM_MOUSELEAVE: {
			if( hover_on ) {
				hover_on = false;
				DestroyWindow( hwnd_hover );
				ReleaseCapture();
			}
			return(0);
		}
		case WM_MOUSEWHEEL: {
		// We use the mousewheel to scroll:
			// Wheel info:
			WORD fwKeys = LOWORD(wParam);
			short zDelta = (short)HIWORD(wParam);
			SHORT mx = LOWORD(lParam), my = HIWORD(lParam);

			// Use the following code to obtain the horizontal and vertical position:
			short xPos = LOWORD(lParam); 
			short yPos = HIWORD(lParam);

			if( zDelta < 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEDOWN,0), 0L);
			} else if( zDelta > 0 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKEWPARAM(SB_LINEUP,0), 0L);
			}
			SendMessage(hWnd, WM_MOUSEMOVE, 0, MAKELPARAM(mx,my) );
			return(0); // An application should return 0 if it processes WM_MOUSEWHEEL.
		}
		case WM_KEYDOWN: {
			int entries_that_fit_on_screen = (rcClient_result.bottom-rcClient_result.top)/(entry_height);
			int end_entry = start_entry + min( entries_that_fit_on_screen, edict_results.size()-1-start_entry );

			WORD wScrollNotify = -1;
			switch (wParam) {
				// Handle the keys for scrolling:
				case VK_UP: {
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( entry_selected > 0 ) {
						if( entry_selected == start_entry ) {	wScrollNotify = SB_LINEUP;	}
						entry_selected--;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_LINEUP;
					}
					break;
				}
				case VK_DOWN: {
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( entry_selected < edict_results.size()-1 ) {
						if( entry_selected == end_entry-1 ) {	wScrollNotify = SB_LINEDOWN;	}
						entry_selected++;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_LINEDOWN;
					}
					break;
				}
				case VK_PRIOR: {//	wScrollNotify = SB_PAGEUP;		break;	}
					// If an entry is selected, move it up; otherwise, just move the scrollbar:
					if( entry_selected >= 0 ) {
						if( entry_selected > 0+entries_that_fit_on_screen-1 ) {
							entry_selected -= entries_that_fit_on_screen;
						} else {
							entry_selected = 0;
						}
						wScrollNotify = SB_PAGEUP;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_PAGEUP;
					}
					break;
				}
				case VK_NEXT: {//	wScrollNotify = SB_PAGEDOWN;	break;	}
					// If an entry is selected, move it down; otherwise, just move the scrollbar:
					if( entry_selected >= 0 ) {
						if( entry_selected < (int)edict_results.size()-entries_that_fit_on_screen ) {
							entry_selected += entries_that_fit_on_screen;
						} else {
							entry_selected = edict_results.size()-1;
						}
						wScrollNotify = SB_PAGEDOWN;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_PAGEDOWN;
					}
					break;
				}
				case VK_HOME: {//ScrollNotify = SB_TOP;			break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( entry_selected >= 0 ) {
						if( entry_selected != 0 ) {
							entry_selected = 0;
						}
						wScrollNotify = SB_TOP;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_TOP;
					}
					break;
				}
				case VK_END: {//wScrollNotify = SB_BOTTOM;		break;	}
					// If an entry is selected, move it to the top; otherwise, just move the scrollbar:
					if( entry_selected >= 0 ) {
						if( entry_selected != edict_results.size()-1 ) {
							entry_selected = edict_results.size()-1;
						}
						wScrollNotify = SB_BOTTOM;
						RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
					} else {
						wScrollNotify = SB_BOTTOM;
					}
					break;
				}
				// Switch windows:
				case VK_TAB: {
					SetFocus( hwnd_searchedit );
					break;
				}
				// Select the top one:
				case VK_RETURN: {
				/* If an entry is selected already, treat it as a double-click.
				   Otherwise, select the top entry on the screen. */

					// If there's already a search going on, stop.
					if( thread_searching ) {  return(0);  }

					if( edict_results.size() > 0 ) {
						// If there's no selected entry, select the top one.
						if( entry_selected == -1 ) {
							entry_selected = start_entry;

							RedrawWindow( hWnd, NULL, NULL, RDW_INVALIDATE );
						} else {
							// Do the double-click action on it.
							// Clear the kanjidic results.
							// Add only the kanji that apply to what was double-clicked:
							kanjidic_results.clear();
							for( int w = 0; w < edict_results[entry_selected].word.size(); w++ ) {
								Uint16 character = edict_results[entry_selected].word[w];
								// If it's not hiragana or katakana, then it could be kanji:
								if( !( IsHiragana(character) || IsKatakana(character) ) ) {
									KANJIDIC_Index_Entry *index = NULL;
									for( int k = 0; k < kanjidic_index_entries_size; k++ ) {
										if( kanjidic_index_entries[k].kanji == character ) {
											index = &kanjidic_index_entries[k];
											break;
										}
									}
									if( index != NULL ) {
										// Now we want to make sure that we haven't already added it:
										bool okaytoadd = true;
										for( int l = 0; l < kanjidic_results.size(); l++ ) {
											if( kanjidic_results[l].kanji == index->kanji ) {
												okaytoadd = false;
												break;
											}
										}
										// If it's new, then add it:
										if( okaytoadd ) {
											KANJIDIC_Entry entry;
												LoadKANJIDICInfoInto( entry, index );
											kanjidic_results.push_back( entry );
										}
									}
								} // if it's probably kanji
							}// for w = 0 --> word_size

							// Resize & redraw the kanji window:
							KanjiWindow_SetUpForNewResults();



							// Set the text of the search edit control to the word:
							SETTEXTEX st;
								st.flags = ST_KEEPUNDO;
								st.codepage = 1200; // 1200 == unicode.
							SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)edict_results[entry_selected].word.c_str() );

							// Select the text:
							SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						}
					}
					break;
				}
			}
 
			if( wScrollNotify != -1 ) {
				SendMessage(hWnd, WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L);
				SendMessage(hWnd, WM_MOUSEMOVE, 0, MAKELPARAM(last_mx,last_my) );
			}
			return(0);
		}// case WM_KEYDOWN
		case WM_CHAR: {
			switch( wParam ) {
				case L'\t': {
					SetFocus( hwnd_searchedit );
					break;
				}
			}
			return(0);
		}// case WM_CHAR
	}
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateResult_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS wndClass; 
	   //wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.style          = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
		wndClass.lpfnWndProc    = WndProc_Result_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("ResultWindow");
   
   RegisterClass(&wndClass);
   
   HWND hWnd = CreateWindowEx(
		WS_EX_CLIENTEDGE,
      TEXT("ResultWindow"),   // window class name
      TEXT("(ResultWindow)"),  // window caption
      WS_CHILD| WS_VISIBLE | WS_VSCROLL, // window style
      x,y, // (x,y)
      w,h, // (w,h)
      hwndParent,                     // parent window handle
      NULL,                     // window menu handle
      g_hinst,                // program instance handle
      NULL);                    // creation parameters
	  
   //ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////