// Search.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_H
#define SEARCH_H
///////////////////////////////////////////////////////////
#include <StringExtras\StringExtras.h>
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringStartsWith( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		for( int i = 0; i < text.size(); i++ ) {
			if( str[i] != text[i] ) {
				return(false);
			}
		}
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
bool StringEndsWith( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		for( int i = 0; i < text.size(); i++ ) {
			if( str[str.size()-1-i] != text[text.size()-1-i] ) {
				return(false);
			}
		}
		return(true);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringHasInIt( wstring &str, wstring &text ) {
	if( str.size() >= text.size() ) {
		bool matched = false;
		for( int i = 0; i < text.size(); i++ ) {
			if( str[i] == text[0] ) {
				matched = true;
				for( int j = 1; j < text.size(); j++ ) {
					if( i+j < str.size() ) {
						if( str[i+j] != text[j] ) {
							matched=false;
							break;
						}
					} else {
						matched=false;
						break;
					}
				}
				if( matched ) {
					return(true);
				}
			}
		}
		return(false);
	} else {
		return(false);
	}
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
bool StringsMatch( wstring &entry_text, wstring &text ) {
	bool addit = false;

	if( wc_wcsncmp( text.c_str(), entry_text.c_str(), WC_EXACT, WC_MATCHCASE,WC_WILDCARDS_ON ) == WC_MATCH ) {
		addit = true;
	}

	return(addit);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
// Search return codes:
enum {
	SEARCH_STATUS_OKAY = 0,
	SEARCH_STATUS_TRUNCATED = 1,
};
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This searches through all active dictionary index
	files for the kana (or the word, depending on the search
	method).
		When if finds a match, it loads all of the information
	from the EDICT into the edict_results vector.
*/
int SearchFor( wstring text ) {
	int search_status = 0;

	if( search_conversion_kana == SEARCH_CONVERSION_KANA_EQUAL ) {
		text = ToKatakana( text );
	}

	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		if( !dictionary->on ) {  continue;  }

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		// Go through all the entries:
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				break;
			}

			wstring entry_kana = dictionary->index_entries[i].kana;
			if( search_conversion_kana == SEARCH_CONVERSION_KANA_EQUAL ) {
				entry_kana = ToKatakana( entry_kana );
			}

			bool addit = false;
			// If we are NOT forcing to look only a words (no kana),
			if( search_forcewordsearch == 0 ) {
				// Find out if we should search through kana or words:
				if( IsKana_WildcardsOkay(text) ) {
					addit = StringsMatch( entry_kana, text );
				} else {
					addit = StringsMatch( wstring(dictionary->index_entries[i].word), text );
				}
			} else {
			// If we ARE forcing a word-search, then do it:
				addit = StringsMatch( wstring(dictionary->index_entries[i].word), text );
			}

			if( addit ) {
				EDICT_Entry entry;
				LoadInfoInto( dictionary, entry, dictionary->index_entries[i] );
					SetInfo( entry );
				edict_results.push_back( entry );
			}
		}// for each entry in the dictionary.
	}// for loop for dictionaries.
	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/*		This is the English SearchFor() variation.  It manually
	loads every entry of every dictionary and checks the
	meaning string to see if there's a match.
*/
int SearchFor_English( wstring wtext ) {
	string text = ToString(wtext);
	text = ToLower( text );
	vector<string> english_words;


	// Fullsearch searches for entries whose definitions are THAT word.
	bool fullsearch = false;
	string fullword;
	if( text[0] == '\"' && text[text.size()-1] == '\"' ) {
		fullword = text.substr(1,text.size()-2 );
		fullsearch = true;
	}


	if( !fullsearch ) {
	// If we're not using a full search, then break the search query down
	// into individual words.
		string word;
		bool quoting = false;
		for( int i = 0; i < text.size(); i++ ) {
			// If it's a letter, add it to the current word:
			if( text[i] >= 'A' && text[i] <= 'Z' || text[i] >= 'a' && text[i] <= 'z' ) {
				word += text[i];
				if( i == text.size()-1 && word.size() > 0 ) {
					english_words.push_back(word);
				}
			} else if( text[i] == '\"' ) {
				if( quoting ) {
					if( word.size() > 0 ) {
						english_words.push_back(word);
						word = "";
					}
					quoting = false;
				}
				else {  quoting = true;  }
			} else {
				if( quoting ) {
					word += text[i];
					if( i == text.size()-1 && word.size() > 0 ) {
						english_words.push_back(word);
					}
				}
				if( !quoting && word.size() > 0 ) {
					english_words.push_back(word);
					word = "";
				}
			}
		}
		fwprintf(g_flogu,L"English search text: %S\n",text.c_str() );
		for( i = 0; i < english_words.size(); i++ ) {
			fwprintf(g_flogu,L"   %S\n",english_words[i].c_str() );
		}
	} // if( !fullsearch )...




	//////////////////////////////
	///   COMMENCE SEARCHING   ///
	//////////////////////////////


	int search_status = 0;

	// Go through all the dictionaries:
	for( int d = 0; d < edict_dictionaries.size(); d++ ) {
		Dictionary_EDICT *dictionary = edict_dictionaries[d];
		if( !dictionary->on ) {  continue;  }

		Uint16 statustext[1024];
			wsprintf(statustext,L"Searching in dictionary '%S'...",dictionary->displayname.c_str() );
		SetStatusBarText(statustext);

		FILE *fedict = fopen( dictionary->filename.c_str(),"rb");

		// Go through all the entries:
		for( int i = 0; i < dictionary->index_entries_size; i++ ) {
			if( edict_results.size() == search_results_max ) {
				search_status |= SEARCH_STATUS_TRUNCATED;
				break;
			}

			// Go through the english words vector, and if the entry has the word, add it.
			// (Remember to apply search rules...)

			bool okaytoadd = true;

			// Get the full line:
			char *line = GetEdictLine( fedict, dictionary->index_entries[i] );
			if( !line ) {  fwprintf(g_flogu,L"No line!\n");  continue;  }
			// Get the meaning from the line:
			char *original_meaning = strchr(line,'/');
			if( !original_meaning ) {  fwprintf(g_flogu,L"No meaning!\n");  continue;  }
			// Convert it to lower case:
			strlwr( original_meaning );


			if( fullsearch ) {
			///////////////////////
			///   FULL SEARCH   ///
			///////////////////////
				//fwprintf(g_flogu,L"String: %S\n",original_meaning );

				vector<string> meanings;

				// Get the meanings:
				string strline = original_meaning;
				int pos = strline.find( '/',0 );
				strline = strline.substr( pos );
				while( strline.size() > 0 ) {
					if( strline[0] == L'/' ) {
						pos = strline.find( '/',1 );
						if( pos != -1 ) {
							string strmeaning = strline.substr( 1,pos-1 );
								meanings.push_back( strmeaning );
							strline = strline.substr( pos,-1 );
						} else {
							break;
						}
					} else {  break;  }
				}

				// Cut off any superficial text for each meaning:
				// (exp) (n) happy (as in feeling)
				//    is changed to:
				// happy
				for( int i = 0; i < meanings.size(); i++ ) {
					bool parentheses = false;
					int start = 0;
					while( parentheses || !( meanings[i][0+start]>='a'&&meanings[i][0+start]<='z' ) ) {
						if( start == meanings[i].size()-1 ) {  break;  }
						if( meanings[i][0+start] == '(' ) {
							parentheses = true;
						} else if( meanings[i][0+start] == ')' ) {
							parentheses = false;
						}
						start++;
					}

					parentheses = false;
					int end = 0;
					while( parentheses || !( meanings[i][meanings[i].size()-1-end]>='a'&&meanings[i][meanings[i].size()-1-end]<='z' ) ) {
						if( end == meanings[i].size()-1 ) {  break;  }
						if( meanings[i][meanings[i].size()-1-end] == ')' ) {
							parentheses = true;
						} else if( meanings[i][meanings[i].size()-1-end] == '(' ) {
							parentheses = false;
						}
						end++;
					}

					if( meanings[i].size() > end+start ) { 
						meanings[i] = meanings[i].substr(start,meanings[i].size()-end);
					}
					//fwprintf(g_flogu,L"   Mod: %S\n",meanings[i].c_str() );
				}


				okaytoadd = false;
				for( i = 0; i < meanings.size(); i++ ) {
					if( strcmp( meanings[i].c_str(), fullword.c_str() ) == 0 ) {
						okaytoadd = true;
						break;
					}
				}


			///////////////////////////
			///   END FULL SEARCH   ///
			///////////////////////////
			} else {
			/////////////////////////
			///   GOOGLE SEARCH   ///
			/////////////////////////
				// Go through the English search words:
				for( int w = 0; w < english_words.size(); w++ ) {
					// If we've already failed, then stop.
					if( !okaytoadd ) {  break;  }

					// Restore the meaning:
					char *meaning = original_meaning;

					char *found = NULL;
					while( true ) {
						// This is where the match is:
						found = strstr( meaning,english_words[w].c_str() );
						if( !found ) {
							okaytoadd = false;
							break;
						} else {
							// If we're searching for the whole word, the text before and after
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_WHOLE ) {
								bool beginningok = true;
								bool endok = true;
								if( found-meaning > 0 ) {
									char c = meaning[(found-meaning)-1];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										beginningok = false;
									}
								}
								if( strlen(found) > english_words[w].size() ) {
									char c = found[english_words[w].size()];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										endok = false;
									}
								}

								// If both are okay, get out of the loop and move on to the next word:
								if( beginningok && endok ) {
									break;
								}
							}

							// If we're searching for the beginning of the word, then the text before
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_START ) {
								if( found-meaning > 0 ) {
									char c = meaning[(found-meaning)-1];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										;
									} else {
										// OKAY!!
										break;
									}
								}
							}

							// If we're searching for the end of the word, then the text after
							// must be non-letter characters.
							if( search_style_location == SEARCH_STYLE_END ) {
								if( strlen(found) > english_words[w].size() ) {
									char c = found[english_words[w].size()];
									if( c>='a'&&c<='z' ) {// || c>='A'&&c<='Z' ) {
										;
									} else {
										// OKAY!!
										break;
									}
								}
							}

							// If we're searching for the word anywhere, just get it over with:
							if( search_style_location == SEARCH_STYLE_MIDDLE ) {
								break;
							}
						}

						if( strlen(found)>1 ) {
							meaning = found+1;
						} else {
							okaytoadd = false;
							break;
						}
					} // while(true);

					if( !found ) {
						okaytoadd = false;
					}
				} // for english words...
			/////////////////////////////
			///   END GOOGLE SEARCH   ///
			/////////////////////////////
			} // if( !fullsearch )...

			// Clean up.
			delete [] line;

			// If we had a match, add it:
			if( okaytoadd ) {
				EDICT_Entry entry_empty;
				LoadInfoInto( fedict, entry_empty, dictionary->index_entries[i] );
					SetInfo( entry_empty );
				edict_results.push_back( entry_empty );
			}

		}// for each entry in the dictionary.

		// Close the dictionary and move on:
		fclose( fedict );

	}// for loop for dictionaries.

	return( search_status );
}//*/
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////
