// Search_Window.h | by Doug Manley
///////////////////////////////////////////////////////////
#ifndef SEARCH_WINDOW_H
#define SEARCH_WINDOW_H
///////////////////////////////////////////////////////////
// For the Rich Edit controls:
#include <Richedit.h>

// For the Search Window's Rebar:
#include "Search_Rebar.h"

// For converting ROMAji to kana:
#include "Conversion.h"

// For converting KANJIDIC's kanji to unicode:
#include "JISToUNI2.h"

// For copying text based on the combobox:
#include "CopyPaste.h"
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
LRESULT CALLBACK WndProc_Search_Window(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	switch( message ) {
		case WM_CREATE: {
			hwnd_searchrebar = CreateSearch_Rebar( hWnd );


			RECT rcToolbar;
				GetWindowRect( hwnd_searchrebar, &rcToolbar );

			RECT rc;
				GetClientRect( hWnd, &rc );
			rc.top += rcToolbar.bottom - rcToolbar.top;

			// Load the rich-edit library.
			//    We want to use Rich Edit 3.0 (which is in the
			// file "riched20.dll".
#ifdef VERSION_NT
			// In the NT version, load it only if we have to.
			if( !GetModuleHandle(L"RICHED20.DLL") ) {
				HMODULE hmod = LoadLibrary(L"RICHED20.DLL");
#else
			// In the non-NT version, load the one in the directory.
			{
				// Load the one in the directory to be safe:
				LPTSTR cmdline = GetCommandLine();
				fwprintf(g_flogu,L"cmdline: %s\n",cmdline );
				wstring path = cmdline;
				int pos = path.find(L'\"');
				if( pos >= 0 ) {
					path = path.substr( pos+1 );

					pos = path.find_last_of(L'\\');
					if( pos >= 0 ) {
						path = path.substr( 0, pos );
					}
				}
				fwprintf(g_flogu,L"path: %s\n",path.c_str() );
				wstring str = path + L"\\RICHED20.DLL";
				HMODULE hmod = LoadLibrary(str.c_str());
#endif
				if( !hmod ) {
					MessageBox(hWnd,L"LoadLibrary(RICHED20.DLL) failed.",L"Error",MB_OK );
					fwprintf(g_flogu,L"ERROR: Library 'RICHED20.DLL' not loaded.\n");
				} else {
					fwprintf(g_flogu,L"Library 'RICHED20.DLL' loaded sucessfully.\n");
				}
			}

			hwnd_searchedit = CreateWindowEx(
				WS_EX_CLIENTEDGE,
				RICHEDIT_CLASS, NULL,
            WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | 
            ES_LEFT | ES_MULTILINE,//|ES_NOHIDESEL,// | ES_MULTILINE|ES_WANTRETURN,// | ES_AUTOVSCROLL, 
            0,0, 0,0,  // set size in WM_SIZE message 
            hWnd,        // parent window 
            (HMENU)WID_SEARCH_EDIT, // edit control ID 
            (HINSTANCE)g_hinst, 
            NULL
			);
			SetWindowPos( hwnd_searchedit, 0, rc.left,rc.top, rc.right-rc.left,rc.bottom-rc.top, SWP_NOZORDER|SWP_SHOWWINDOW );
			// IME stuff:
			//SendMessage( hwnd_searchedit, EM_SETIMESTATUS, EMSIS_COMPOSITIONSTRING, EIMES_GETCOMPSTRATONCE );
			// Set the margins:
			SendMessage( hwnd_searchedit, EM_SETMARGINS, EC_LEFTMARGIN,2 );

			// Rich-edit options:
			// To receive EN_MSGFILTER notifications for events: 
			SendMessage( hwnd_searchedit, EM_SETEVENTMASK, 0, ENM_KEYEVENTS|ENM_MOUSEEVENTS );
			// Set it so that it's just normal, plain text:
			SendMessage( hwnd_searchedit, EM_SETTEXTMODE, TM_PLAINTEXT, 0 );

			// Basically, set the font and stuff:
			CHARFORMAT2 charformat;
				charformat.cbSize = sizeof(CHARFORMAT2);
				charformat.dwEffects = 0;
				charformat.bPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
				charformat.dwMask = CFM_FACE | CFM_SIZE | CFM_COLOR | CFM_CHARSET | CFM_WEIGHT;
				// CFM_FACE:
				wcscpy(charformat.szFaceName, ToWString( registry.GetSTRING(".\\Search\\Font\\Font") ).c_str() );
				// CFM_SIZE:
				charformat.yHeight = registry.GetINT(".\\Search\\Font\\Size") * 20; // NOTE: 20 is a constant which converts 1/1440 of an inch to 1/72 of an inch (pt size).
				// CFM_COLOR:
				charformat.crTextColor = registry.GetINT(".\\Search\\Font\\Color");
				// CFM_CHARSET:
				charformat.bCharSet = DEFAULT_CHARSET;//SHIFTJIS_CHARSET;//DEFAULT_CHARSET;
				// CFM_WEIGHT:
				charformat.wWeight = FW_REGULAR;
			SendMessage( hwnd_searchedit, EM_SETCHARFORMAT, (WPARAM)SCF_ALL, (LPARAM)&charformat );

			// Set the background color of the window:
			SendMessage( hwnd_searchedit, EM_SETBKGNDCOLOR, FALSE, registry.GetINT(".\\Search\\WindowBG") );


			// Put in some text:
			Uint16 starttext[1024];
			if( registry.GetBOOL(".\\Search\\ShowInitialText") ) {
				swprintf(starttext,L"\
   To search for Japanese text, just type either the Japanese or the ROMAji and press Enter.\n\
\n\
   To search for English text, type 'EN:' followed by the text and press Enter.  For example,\n\
EN:hair\n\
will search for any entries with 'hair' in the meanings.\n\
   English searches are much like a Google search.  However, if quotes are the FIRST THING after the EN:, then it will search for results exactly matching your text.\n\
   Go ahead, test it:\n\
EN:hair\n\
EN:\"hair\"\n\
\n\
Check for updates & important news often: Options-->Check for Updates.\
");
			} else {
				swprintf(starttext,L"Search for...");
			}
			SendMessage( hwnd_searchedit, WM_SETTEXT, 0, (LPARAM)starttext );


			//HIMC himc = ImmGetContext(hwnd_searchedit);
			//ImmSetOpenStatus( himc, TRUE );
			//ImmSetConversionStatus( himc, IME_CMODE_KATAKANA, IME_SMODE_AUTOMATIC );
			//SendMessage( hwnd_searchedit, WM_INPUTLANGCHANGE, SHIFTJIS_CHARSET, 0x0411 );

			return(0);
		}
		case WM_SIZE: {
			int neww = LOWORD(lParam), newh = HIWORD(lParam);

			// Fix the toolbar window automatically:
			//if( neww > 800 ) {
				SendMessage( hwnd_searchrebar, WM_SIZE, wParam,lParam );
			//}
			// (I don't want it to mess up or anything)

			// Fix the edit window:
			RECT rcToolbar;
				GetWindowRect( hwnd_searchrebar, &rcToolbar );
			RECT rcWindow;
				GetWindowRect( hWnd, &rcWindow );
			SetWindowPos( hwnd_searchedit, NULL,
				0,rcToolbar.bottom-rcToolbar.top,
				neww, newh-(rcToolbar.bottom-rcToolbar.top-1),
				SWP_NOZORDER|SWP_SHOWWINDOW
			);
			return(0);
		}
		case WM_NOTIFY: {
			// If the notification is from the edit-control:
			if( wParam == WID_SEARCH_EDIT ) {
				NMHDR *hdr = (NMHDR*)lParam;
				switch( hdr->code ) {
					case EN_MSGFILTER: {
						MSGFILTER *mf = (MSGFILTER*)lParam;
						switch( mf->msg ) {
							case WM_MOUSEMOVE: {
							/* If the mouse moves over the edit window (in the search window),
							   then select all of the text automatically. */
								HWND hwnd_focus = GetFocus();
								if( GetForegroundWindow()==hwnd_mainwindow && hwnd_focus != hwnd_searchedit ) {
									HWND hwnd_old = SetFocus( hwnd_searchedit );
									if( hwnd_old == NULL ) {
										fwprintf(g_flogu,L"SetFocus(): ERROR: %u\n",GetLastError() );
									}
									SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
								}
								break;
							}
							case WM_RBUTTONDOWN: {
							/* If the right mouse button is clicked, bring up a menu:
									Undo [action]
									Copy
									Cut
									Paste
								(It uses SendMessage() to do all of these). */
								SHORT mx = LOWORD(mf->lParam), my = HIWORD(mf->lParam);

								RECT rcWindow;
									GetWindowRect(hdr->hwndFrom,&rcWindow);

								/////////////////////////
								///   Make the menu   ///
								/////////////////////////
								HMENU hmenu = CreatePopupMenu();
								// UNDO:
									wstring strundo = L"&Undo";
									int undoflags = 0;
									int undoaction = SendMessage( hdr->hwndFrom, EM_GETUNDONAME, 0,0 );
									switch( undoaction ) {
										case UID_TYPING: {  strundo += L" typing"; break;  }
										case UID_DELETE: {  strundo += L" delete"; break;  }
										case UID_DRAGDROP: {  strundo += L" drag and drop"; break;  }
										case UID_CUT: {  strundo += L" cut"; break;  }
										case UID_PASTE: {  strundo += L" paste"; break;  }
									} 
									if( SendMessage( hdr->hwndFrom,EM_CANUNDO, 0,0 ) == 0 ) {
										undoflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|undoflags, 1, strundo.c_str() );
										AppendMenu( hmenu, MF_SEPARATOR, NULL,NULL );
								// COPY, CUT:
									int copyflags = 0;
									if( SendMessage( hdr->hwndFrom,EM_SELECTIONTYPE, 0,0 ) == SEL_EMPTY ) {
										copyflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|copyflags, 2, L"&Copy" );
									AppendMenu( hmenu, MF_STRING|copyflags, 3, L"C&ut" );
								// PASTE:
									int pasteflags = 0;
									if( SendMessage( hdr->hwndFrom,EM_CANPASTE, 0, 0 ) == 0 ) {
										pasteflags |= MF_GRAYED;
									}
									AppendMenu( hmenu, MF_STRING|pasteflags, 4, L"&Paste" );								

								///////////////////////////
								///   Handle the Menu   ///
								///////////////////////////
								int result = TrackPopupMenuEx( hmenu, TPM_LEFTALIGN|TPM_LEFTBUTTON | TPM_RETURNCMD, rcWindow.left+mx,rcWindow.top+my, hWnd, NULL );
								if( result == 1 ) {
								// UNDO:
									SendMessage( hdr->hwndFrom, WM_UNDO, 0,0 );
								} else if( result == 2 ) {
								// COPY:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
								} else if( result == 3 ) {
								// CUT:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
									SendMessage( hdr->hwndFrom, EM_REPLACESEL, TRUE, (LPARAM)L"" );
								} else if( result == 4 ) {
								// PASTE:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_PASTE,0), 0 );
								}

								return(0);
							}
							case WM_KEYDOWN: {
								if( mf->wParam == VK_RETURN ) {
								/* Do a SEARCH when he presses ENTER.
									However, if SHIFT is down, then insert a line break. */
									// If shift is down, just stop:
									if( GetKeyState(VK_SHIFT)&0x8000 ) {  return(0);  }
									// Otherwise, do the search:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_SEARCH_GO,0), 0 );
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								} else if( mf->wParam == VK_v && GetKeyState(VK_CONTROL)&0x8000 ) {
								// If it's CTRL+V (paste):
								// Convert the text to Unicode and paste it.
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_PASTE,0), 0 );
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								} else if( mf->wParam == VK_c && GetKeyState(VK_CONTROL)&0x8000 ) {
								// If it's CTRL+C (copy):
								// Copy the text to whatever format is in the box:
									SendMessage( hWnd, WM_COMMAND, MAKEWPARAM(ID_ENCODE_COPY,0), 0 );
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								}
								break;
							}
							case WM_CHAR: {
								if( mf->wParam == L'\t' ) {
								// Switch windows when 'tab' is pressed:
									SetFocus( hwnd_kanjiwindow );
									mf->wParam = 0;
									mf->msg = 0;
									return(0);
								}
								break;
							}
						}// switch(mf->msg)
						break;
					} // case EN_MSGFILTER
				} // switch hdr->code

			} else if( wParam == WID_SEARCH_REBAR ) {
				NMHDR *hdr = (NMHDR*)lParam;
				switch( hdr->code ) {
					case RBN_AUTOSIZE: {
						//MessageBox(NULL,L"DING",L"!",MB_OK );
						NMRBAUTOSIZE *nas = (NMRBAUTOSIZE*)lParam;
						nas->fChanged = FALSE;
						break;
					}
				}
			} else {
				HandleTooltips(lParam);
			}
			return(0);
		}
		case WM_COMMAND: {
			/* wParam
					The high-order word specifies the notification code if the message is from a control.
					If the message is from an accelerator, this value is 1.
					If the message is from a menu, this value is zero. 
					
					The low-order word specifies the identifier of the menu item, control, or accelerator. 

				lParam
					Handle to the control sending the message if the message is from a control. Otherwise, this parameter is NULL. 
			//*/

			// For the Edit Control:
			// lParam is who sent it (hande).
			// LOWORD(wParam) is who sent it (id).
			// HIWORD(wParam) is the notification message.

			switch( LOWORD(wParam) ) {
				case ID_SEARCH_START: {
					search_style_location = SEARCH_STYLE_START;
					//fprintf(g_flog,"search style :: SEARCH_STYLE_START\n");
					break;
				}
				case ID_SEARCH_MIDDLE: {
					search_style_location = SEARCH_STYLE_MIDDLE;
					//fprintf(g_flog,"search style :: SEARCH_STYLE_MIDDLE\n");
					break;
				}
				case ID_SEARCH_END: {
					search_style_location = SEARCH_STYLE_END;
					//fprintf(g_flog,"search style :: SEARCH_STYLE_END\n");
					break;
				}
				case ID_SEARCH_WHOLE: {
					search_style_location = SEARCH_STYLE_WHOLE;
					//fprintf(g_flog,"search style :: SEARCH_STYLE_WHOLE\n");
					break;
				}

				case ID_MATCHSUBSTRINGS: {
					search_matchsubstrings ^= 1;
					break;
				}
				case ID_FORCEWORDSEARCH: {
					search_forcewordsearch ^= 1;
					break;
				}

				case ID_SEARCH_CONVERSION_ROMAJI: {
					search_conversion_romaji ^= 1;
					break;
				}
				case ID_SEARCH_CONVERSION_KANA: {
					search_conversion_kana ^= 1;
					break;
				}

				case ID_SEARCH_CONVERSION_SHOW: {
					search_conversion_show ^= 1;
					break;
				}

				case WID_SEARCH_COMBOBOX: {
					switch( HIWORD(wParam) ) {
						case CBN_SELENDOK: {
							// If he clicks on <<Something>>, make the selection the one under that.
							int encoding = SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 );
							int index = -1;
							for( int i = 0; i < encoding_array_size; i++ ) {
								if( encoding_array[i].combobox_index == encoding ) {
									index = i;
									break;
								}
							}
							if( encoding_array[i].iconv_string == NULL ) {
								SendMessage( hwnd_searchrebar_combobox_encode,CB_SETCURSEL, (WPARAM)(encoding+1), 0 );
							}
							break;
						}
					}
					break;
				}

				case ID_ENCODE_PASTE: {
				// Convert the text (from the format in the combobox) to Unicode (for the program).
				// Note: this copies the text on the Clipboard and pastes it in place of the selection.
				/* IMPORTANT:
					The clipboard is interesting in some respects.  For example if I copy the word:
					   T[syatu]
					most programs will copy the text in Unicode format for Windows.  Now,
					because it's unicode, the program will put it in CF_UNICODETEXT format.  BUT,
					also, to be nice to non-unicode programs, it will ALSO put it in CF_TEXT format,
					looking like:
					   T???
					This way, CF_TEXT CANNOT be used reliably 
				*/
					// Get the desired encoding (index):
					int encoding = SendMessage( hwnd_searchrebar_combobox_encode,CB_GETCURSEL, 0,0 );
					int index = -1;
					for( int i = 0; i < encoding_array_size; i++ ) {
						if( encoding_array[i].combobox_index == encoding ) {  index = i;  break;  }
					}

					wstring newtext;

					// As long as the index is valid, convert it:
					if( index >= 0 ) {						
						// index 1 is 'Normal Unicode' (no processing).
						if( index == 1 && IsClipboardFormatAvailable(CF_UNICODETEXT) ) {
							// Just make sure that it's plain-text:
							// Get the text that he wants to paste:
							OpenClipboard( NULL );
								HGLOBAL hglobal = GetClipboardData( CF_UNICODETEXT );
								Uint16* paste_text = (Uint16*)GlobalLock(hglobal);
									//wcscat( paste_text, L"\0" );
							CloseClipboard();
	
							newtext = paste_text;
						} else if( IsClipboardFormatAvailable(CF_TEXT) ) {
							// Get the text that he wants to paste:
							OpenClipboard( NULL );
								HGLOBAL hglobal = GetClipboardData( CF_TEXT );
								char* paste_text = (char*)GlobalLock( hglobal );
									//strcat(paste_text,"\0");
							CloseClipboard();
	
							// Convert the text to Unicode so we can display it:
							ConvertEncoding( paste_text,strlen(paste_text), encoding_array[index].iconv_string, "UTF-16LE", newtext );
						} else {
							//MessageBox(hWnd,TEXT("CF_TEXT format is not availabe."),TEXT("Warning"),MB_OK );	
						}
					} else {
						fwprintf(g_flogu,L"ID_ENCODE_PASTE: Error: index out of bounds (index: %i).\n",index );
					}
					// Replace the selection with the new text (now in unicode format):
					//SendMessage( hwnd_searchedit,EM_REPLACESEL, (WPARAM)TRUE, (LPARAM)newtext.c_str() );//*/
					SETTEXTEX st;
						st.flags = ST_KEEPUNDO|ST_SELECTION;
						st.codepage = 1200; // 1200 == unicode.
					SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)newtext.c_str() );
					return(0);
				}
				case ID_ENCODE_COPY: {
				/* Copy the text to the format as specified by the combobox:
				      If the option is 'Normal Unicode', then don't do any conversion and
					just copy it in the CF_UNICODETEXT format.
				*/
					// Get the text from the edit control:
					int sel_start,sel_stop;
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					// If something's selected, copy it:
					if( sel_stop-sel_start > 0 ) {
						Uint16 *tcharsearchtext = new Uint16[sel_stop-sel_start+1];
						SendMessage(hwnd_searchedit,EM_GETSELTEXT,0,(LPARAM)tcharsearchtext );
						wstring searchtext = tcharsearchtext;
							delete [] tcharsearchtext;
					
						// If there's nothing to copy, just stop.
						if( searchtext.size() <= 0 ) {	return(0);	}

						// Use a nice function to do the copying:
						CopyTextUsingComboboxEncoding( searchtext );
					}
					return(0);
				}
				case ID_SEARCH_GO: {
				// Search for the text in the search window:

	
					// If a dictionary is loading in a separate thread, cancel the search.
					// We don't want to be searching for something when the dictionaries
					// are still being written to.
					//    OR
					// If a search is already underway, stop.
					if( thread_searching ) {
						MessageBox(hWnd,L"Please finish the current search first or\nlet the dictionary finish loading.\n",L"Note",MB_OK );
						break;
					}




					// Set the status bar text:
					SetStatusBarText(L"Searching...");

					// Get the text from the edit control:
					int sel_start,sel_stop;
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					// If something isn't selected, select everything:
					if( sel_stop-sel_start <= 0 ) {
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						SendMessage(hwnd_searchedit,EM_GETSEL,(WPARAM)&sel_start,(LPARAM)&sel_stop );
					}

					Uint16 *tcharsearchtext = NULL;
					if( sel_stop-sel_start > 0 ) {
						tcharsearchtext = new Uint16[sel_stop-sel_start+1];
						SendMessage(hwnd_searchedit,EM_GETSELTEXT,0,(LPARAM)tcharsearchtext );
					} else {
						SetStatusBarText(L"(no text to search for)");
						break;
					}
					wstring searchtext = tcharsearchtext;
					if( tcharsearchtext ) {
						delete [] tcharsearchtext;
					}
					if( searchtext.size() == 0 ) {
						break;
					}

					wstring searchtext_old = searchtext;

					// Get rid of end lines and carriage returns:
					while( true ) {
						int pos = searchtext.find(L'\n');
						if( pos >= 0 ) {  searchtext.erase( searchtext.begin()+pos );  }
						else {  break;  }
					}
					while( true ) {
						int pos = searchtext.find(L'\r');
						if( pos >= 0 ) {  searchtext.erase( searchtext.begin()+pos );  }
						else {  break;  }
					}


					// Thread search time!
					THREAD_SEARCH ts;
						ts.search_text = &searchtext;
						ts.mode = TS_NORMAL;
					// Start searching (thread):
					HANDLE hthread = Load_Thread_Search( &ts );

					// Sit around for a bit until:
					// 1) The thread finishes, or
					// 2) He presses ESCAPE.
					enum {  SEARCH_ABORT, SEARCH_DONE, SEARCH_QUIT  };
					int wtf = -1;
					MSG msg;
					while( true ) {
						if( !thread_searching ) {  wtf = SEARCH_DONE;  break;  }

						// Check to see if we have an ESCAPE key message.
						PeekMessage( &msg, NULL, 0,0, PM_REMOVE );
						TranslateMessage(&msg);
						if( msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE ) {  wtf = SEARCH_ABORT;  break;  }
						if( msg.message == WM_QUIT ) {  wtf = SEARCH_QUIT;  break;  }
						DispatchMessage(&msg);

						Sleep( 10L );
					}
					DestroyWindow(hwnd_searching);
					if( wtf != SEARCH_DONE ) {
					//////////////////////////
					///   SEARCH ABORTED   ///
					//////////////////////////
						TerminateThread( hthread, 0L );
						CloseHandle(hthread);
						thread_searching = false;

							// Kanji:
							kanjidic_results.clear();
							// Go through the edict results looking for kanji:
							for( int i = 0; i < edict_results.size(); i++ ) {
								KanjiWindow_SearchForKanjiIn( edict_results[i].word );
							}

						// Select the text for ease.
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ) {
							SendMessage( hwnd_searchedit, EM_SETSEL, 3,-1 );
						}

						// Set up & redraw the kanji window:
						// (this also sorts them).
						KanjiWindow_SetUpForNewResults();

						// Set up & redraw the result window:
						// (this also sorts them).
						ResultWindow_SetUpForNewResults();

						// Update the status bar text:
						TCHAR tsearchbar[400];
						swprintf(tsearchbar,L"Search aborted (%i results)",edict_results.size()	);
						SetStatusBarText( tsearchbar );


						if( wtf == SEARCH_QUIT ) {
							SendMessage(hwnd_mainwindow,WM_QUIT,0,0);
							return(0);
						}
						break;
					} else {
					/////////////////////////////
					///   SEARCH SUCCESSFUL   ///
					/////////////////////////////
						CloseHandle(hthread);

						int search_status = thread_search_exitcode;
						searchtext = thread_search_text;

						// Set up & redraw the kanji window:
						// (this also sorts them).
						KanjiWindow_SetUpForNewResults();

						// Set up & redraw the result window:
						// (this also sorts them).
						ResultWindow_SetUpForNewResults();


						if( !search_conversion_show ) {
							searchtext = searchtext_old;
						}
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ) {
							searchtext = searchtext_old;
						}


						// Replace the selection with the search text:
						//SendMessage( hwnd_searchedit, EM_REPLACESEL, (WPARAM)TRUE, (LPARAM)searchtext.c_str() );
						SETTEXTEX st;
							st.flags = ST_KEEPUNDO|ST_SELECTION;
							st.codepage = 1200; // 1200 == unicode.
						SendMessage( hwnd_searchedit,EM_SETTEXTEX, (WPARAM)&st, (LPARAM)searchtext.c_str() );

						//	Select the text for ease.
						SendMessage( hwnd_searchedit, EM_SETSEL, 0,-1 );
						if( searchtext_old.find(L"EN:") == 0 || searchtext_old.find(L"en:") == 0 ) {
							SendMessage( hwnd_searchedit, EM_SETSEL, 3,-1 );
						}

						// Update the status bar text:
						TCHAR tsearchbar[400];
						swprintf(tsearchbar,L"Search complete (%i results) %s",edict_results.size(),
							search_status&SEARCH_STATUS_TRUNCATED?L"-Results truncated-":L""
						);
						SetStatusBarText( tsearchbar );//*/
					}// if( mode == ... )
					break;
				}
			}
			return(0);
		}//*/
		case WM_ERASEBKGND: {
			return(TRUE);
		}
	}
	return( DefWindowProc(hWnd, message, wParam, lParam) );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
HWND CreateSearch_Window( HWND hwndParent, int x,int y, int w,int h ) {
	WNDCLASS            wndClass; 
	   wndClass.style          = CS_SAVEBITS; //CS_HREDRAW | CS_VREDRAW;
		wndClass.lpfnWndProc    = WndProc_Search_Window;
	   wndClass.cbClsExtra     = 0;
		wndClass.cbWndExtra     = 0;
	   wndClass.hInstance      = g_hinst;
		wndClass.hIcon          = NULL;//LoadIcon(NULL, IDI_APPLICATION);
	   wndClass.hCursor        = NULL;//LoadCursor(NULL, IDC_ARROW);
		wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	   wndClass.lpszMenuName   = NULL;
		wndClass.lpszClassName  = TEXT("SearchWindow");
   
   RegisterClass(&wndClass);
   
   HWND hWnd = CreateWindow(
      TEXT("SearchWindow"),   // window class name
      TEXT("(SearchWindow)"),  // window caption
      WS_CHILD|WS_VISIBLE,// |WS_BORDER/*|WS_OVERLAPPED*//*|WS_SIZEBOX*/,      // window style
		x,y,
		w,h,
		hwndParent, // parent window handle
      NULL, // window menu handle
      g_hinst, // program instance handle
      NULL // creation parameters
	);
	  
   //ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////